<?php
namespace App\Helpers;

use DB;
use App\Models\Statuslabel;

use App\Models\WarrantyAsset;
use App\Models\AssetModel;
use App\Models\Company;
use App\Models\Master\User;
use App\Models\Manufacturer;
use App\Models\Supplier;
use App\Models\Category;
use App\Models\Depreciation;
use App\Models\CustomFieldset;
use App\Models\CustomField;
use App\Models\Component;
use App\Models\Accessory;
use App\Models\Consumable;
use App\Models\Asset;
use App\Models\Admin;
use App\Models\Project;
use App\Models\B2BWarranty;
use App\Models\WarrantyAssetModel;
use Crypt;
use Illuminate\Contracts\Encryption\DecryptException;
//use Illuminate\Contracts\Auth\Access\Gate;
use App\Models\Skill;
use App\Models\Documenttype;
use Auth;

class Helper
{


    /**
     * Simple helper to invoke the markdown parser
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.0]
     * @return String
     */
    public static function parseEscapedMarkedown($str) {
        $Parsedown = new \Parsedown();

        if ($str) {
            return $Parsedown->text(e($str));
        }
    }


    /**
     * The importer has formatted number strings since v3,
     * so the value might be a string, or an integer.
     * If it's a number, format it as a string.
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.0]
     * @return String
     */
    public static function formatCurrencyOutput($cost)
    {
        if (is_numeric($cost)) {
            return number_format($cost, 2, '.', '');
        }
        // It's already been parsed.
        return $cost;
    }


    /**
     * Static colors for pie charts.
     * This is inelegant, and could be refactored later.
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v3.3]
     * @return Array
     */
    public static function chartColors()
    {
        $colors = [
            '#f56954',
            '#00a65a',
            '#f39c12',
            '#00c0ef',
            '#3c8dbc',
            '#d2d6de',
            '#3c8dbc',
            '#3c8dbc',
            '#3c8dbc',

        ];
        return $colors;
    }


    /**
     * Static background (highlight) colors for pie charts
     * This is inelegant, and could be refactored later.
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v3.2]
     * @return Array
     */
    public static function chartBackgroundColors()
    {
        $colors = [
            '#f56954',
            '#00a65a',
            '#f39c12',
            '#00c0ef',
            '#3c8dbc',
            '#d2d6de',
            '#3c8dbc',
            '#3c8dbc',
            '#3c8dbc',

        ];
        return $colors;
    }


    /**
     * Format currency using comma for thousands until local info is property used.
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.7]
     * @return String
     */
    public static function ParseFloat($floatString)
    {
        $LocaleInfo = localeconv();
        $floatString = str_replace(",", "", $floatString);
        $floatString = str_replace($LocaleInfo["decimal_point"], ".", $floatString);
        return floatval($floatString);
    }


    /**
     * Get the list of models in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function modelList()
    {
        $models = AssetModel::with('manufacturer')->get();
        $model_array[''] = trans('general.select_model');
        foreach ($models as $model) {
            $model_array[$model->id] = $model->displayModelName();
        }
        return $model_array;
    }

    /**
     * Get the list of companies in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function companyList()
    {  
        //if (Gate::check('superadmin'))\
        if (app(\Illuminate\Contracts\Auth\Access\Gate::class)->check('superadmin') || app(\Illuminate\Contracts\Auth\Access\Gate::class)->check('admin'))
        {
         $company_list = array('' => trans('general.select_company')) + DB::table('companies')
                ->whereNull('deleted_at')
                ->where('status','=','Active')
                ->orderBy('name', 'asc')
                ->pluck('name', 'id')->toArray();
        }
        else
        {   
           $admin = Admin::find(Auth::user()->id);
           $project_id=explode(',', $admin->project_id);
           $companies_ids = Project::whereIn('id',$project_id)->groupBy('companies_id')->pluck('companies_id'); 
           $company_list = array('' => trans('general.select_company')) + DB::table('companies')
                ->whereNull('deleted_at')
                ->where('status','=','Active')
                ->whereIn('id',$companies_ids)
                ->orderBy('name', 'asc')
                ->pluck('name', 'id')->toArray();
        }
        

       
        return $company_list;
    }
   

    /**
     * Get the list of categories in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function categoryList($category_type = null)
    {
        $categories = Category::orderBy('name', 'asc')
                ->whereNull('deleted_at')
                ->orderBy('name', 'asc');
        if(!empty($category_type))
            $categories = $categories->where('category_type', '=', $category_type);
        $category_list = array('' => trans('general.select_category')) + $categories->pluck('name', 'id')->toArray();
        return $category_list;
    }

    /**
     * Get the list of suppliers in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function suppliersList()
    {
        $supplier_list = array('' => trans('general.select_supplier')) + Supplier::orderBy('name', 'asc')
                ->orderBy('name', 'asc')
                ->pluck('name', 'id')->toArray();
        return $supplier_list;
    }

    /**
     * Get the list of suppliers in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function materialsuppliersList()
    {
        $supplier_list = array('' => trans('general.select_supplier')) + Supplier::orderBy('name', 'asc')
                ->orderBy('name', 'asc')
                ->pluck('name', 'name')->toArray();
        return $supplier_list;
    }


    /**
     * Get the list of status labels in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function statusLabelList()
    {
        $statuslabel_list = array('' => trans('general.select_statuslabel')) + Statuslabel::orderBy('name', 'asc')
                ->pluck('name', 'id')->toArray();
        return $statuslabel_list;
    }


    /**
     * Get the list of locations in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function locationsList()
    {

        $location_list = Location::whereNull('deleted_at')
                ->where('status','=','Active')
                ->orderBy('name', 'asc')
                ->pluck('name', 'id')->toArray();

        return $location_list;
    }


    /**
     * Get the list of manufacturers in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function manufacturerList()
    {
        $manufacturer_list = array('' => trans('general.select_manufacturer')) +
            Manufacturer::orderBy('name', 'asc')
                ->pluck('name', 'id')->toArray();
        return $manufacturer_list;
    }

    /**
     * Get the list of status label types in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function statusTypeList()
    {
        $statuslabel_types = array('' => trans('admin/hardware/form.select_statustype')) + array('undeployable' => trans('admin/hardware/general.undeployable')) + array('pending' => trans('admin/hardware/general.pending')) + array('archived' => trans('admin/hardware/general.archived')) + array('deployable' => trans('admin/hardware/general.deployable'));
        return $statuslabel_types;
    }

    /**
     * Get the list of managers in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function managerList()
    {
        $manager_list = array('' => trans('general.select_user')) +
                        User::where('deleted_at', '=', null)->where('blacklist','!=',1)
                        ->where('status','=','Active')
                        ->orderBy('last_name', 'asc')
                        ->orderBy('first_name', 'asc')->get()
                        ->pluck('full_name', 'id')->toArray();

        return $manager_list;
    }

    /**
     * Get the list of depreciations in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function depreciationList()
    {
        $depreciation_list = ['' => 'Do Not Depreciate'] + Depreciation::orderBy('name', 'asc')
                ->pluck('name', 'id')->toArray();
        return $depreciation_list;
    }

    /**
     * Get the list of category types in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function categoryTypeList()
    {
        $category_types = array('' => '','Network' => 'Network', 'Server' => 'Server', 'Safety and Security' => 'Safety and Security','Audio Visual' => 'Audio Visual','Endpoints'=>'Endpoints','UC'=>'UC','UPS'=>'UPS','PA System'=>'PA System' , 'Data Centre' => 'Data Centre' , 'Passive' => 'Passive' );
        return $category_types;
    }

    /**
     * Get the list of users in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function usersList()
    {
        $users_list =   array( '' => trans('general.select_user')) +
                        Company::scopeCompanyables(User::where('deleted_at', '=', null))
                        ->where('status','=','Active')
                        ->where('show_in_list','=',1)
                        ->orderBy('last_name', 'asc')
                        ->orderBy('first_name', 'asc')->get()
                        ->pluck('full_name', 'id')->toArray();

        return $users_list;
    }

    /**
     * Get the list of assets in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function assetsList()
    {
        $assets_list = array('' => trans('general.select_asset')) + Asset::orderBy('name', 'asc')
                ->whereNull('deleted_at')
                ->pluck('name', 'id')->toArray();
        return $assets_list;
    }

    /**
     * Get the detailed list of assets in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function detailedAssetList()
    {
        $assets = array('' => trans('general.select_asset')) + Company::scopeCompanyables(Asset::with('assignedUser', 'model'), 'assets.company_id')->get()->pluck('detailed_name', 'id')->toArray();
        return $assets;
    }


    /**
     * Get the list of custom fields in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function customFieldsetList()
    {
        $customfields = array('' => trans('admin/models/general.no_custom_field')) + CustomFieldset::pluck('name', 'id')->toArray();
        return  $customfields;
    }

    /**
     * Get the list of custom field formats in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v3.4]
     * @return Array
     */
    public static function predefined_formats()
    {
        $keys=array_keys(CustomField::$PredefinedFormats);
        $stuff=array_combine($keys, $keys);
        return $stuff+["" => trans('admin/custom_fields/general.custom_format')];
    }

    /**
     * Get the list of barcode dimensions
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v3.3]
     * @return Array
     */
    public static function barcodeDimensions($barcode_type = 'QRCODE')
    {
        if ($barcode_type == 'C128') {
            $size['height'] = '-1';
            $size['width'] = '-10';
        } elseif ($barcode_type == 'PDF417') {
            $size['height'] = '-3';
            $size['width'] = '-10';
        } else {
            $size['height'] = '-3';
            $size['width'] = '-3';
        }
        return $size;
    }

    /**
     * Generates a random string
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v3.0]
     * @return Array
     */
    public static function generateRandomString($length = 10)
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }


    /**
     * This nasty little method gets the low inventory info for the
     * alert dropdown
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v3.0]
     * @return Array
     */
    public static function checkLowInventory()
    {
        $consumables = Consumable::with('users')->whereNotNull('min_amt')->get();
        $accessories = Accessory::with('users')->whereNotNull('min_amt')->get();
        $components = Component::with('assets')->whereNotNull('min_amt')->get();

        $avail_consumables = 0;
        $items_array = array();
        $all_count = 0;

        foreach ($consumables as $consumable) {
            $avail = $consumable->numRemaining();
            if ($avail < ($consumable->min_amt) + \App\Models\Setting::getSettings()->alert_threshold) {
                if ($consumable->qty > 0) {
                    $percent = number_format((($consumable->numRemaining() / $consumable->qty) * 100), 0);
                } else {
                    $percent = 100;
                }

                $items_array[$all_count]['id'] = $consumable->id;
                $items_array[$all_count]['name'] = $consumable->name;
                $items_array[$all_count]['type'] = 'consumables';
                $items_array[$all_count]['percent'] = $percent;
                $items_array[$all_count]['remaining']=$consumable->numRemaining();
                $items_array[$all_count]['min_amt']=$consumable->min_amt;
                $all_count++;
            }


        }

        foreach ($accessories as $accessory) {
            $avail = $accessory->numRemaining();
            if ($avail < ($accessory->min_amt) + \App\Models\Setting::getSettings()->alert_threshold) {

                if ($accessory->qty > 0) {
                    $percent = number_format((($accessory->numRemaining() / $accessory->qty) * 100), 0);
                } else {
                    $percent = 100;
                }

                $items_array[$all_count]['id'] = $accessory->id;
                $items_array[$all_count]['name'] = $accessory->name;
                $items_array[$all_count]['type'] = 'accessories';
                $items_array[$all_count]['percent'] = $percent;
                $items_array[$all_count]['remaining']=$accessory->numRemaining();
                $items_array[$all_count]['min_amt']=$accessory->min_amt;
                $all_count++;
            }

        }

        foreach ($components as $component) {
            $avail = $component->numRemaining();
            if ($avail < ($component->min_amt) + \App\Models\Setting::getSettings()->alert_threshold) {
                if ($component->qty > 0) {
                    $percent = number_format((($component->numRemaining() / $component->qty) * 100), 0);
                } else {
                    $percent = 100;
                }

                $items_array[$all_count]['id'] = $component->id;
                $items_array[$all_count]['name'] = $component->name;
                $items_array[$all_count]['type'] = 'components';
                $items_array[$all_count]['percent'] = $percent;
                $items_array[$all_count]['remaining']=$component->numRemaining();
                $items_array[$all_count]['min_amt']=$component->min_amt;
                $all_count++;
            }

        }



        return $items_array;


    }


    /**
     * Check if the file is an image, so we can show a preview
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v3.0]
     * @param File $file
     * @return String | Boolean
     */
    public static function checkUploadIsImage($file)
    {
        $finfo = @finfo_open(FILEINFO_MIME_TYPE); // return mime type ala mimetype extension
        $filetype = @finfo_file($finfo, $file);
        finfo_close($finfo);

        if (($filetype=="image/jpeg") || ($filetype=="image/jpg")   || ($filetype=="image/png") || ($filetype=="image/bmp") || ($filetype=="image/gif")) {
            return $filetype;
        }

        return false;
    }

    /**
     * Walks through the permissions in the permissions config file and determines if
     * permissions are granted based on a $selected_arr array.
     *
     * The $permissions array is a multidimensional array broke down by section.
     * (Licenses, Assets, etc)
     *
     * The $selected_arr should be a flattened array that contains just the
     * corresponding permission name and a true or false boolean to determine
     * if that group/user has been granted that permission.
     *
     * @author [A. Gianotto] [<snipe@snipe.net]
     * @param array $permissions
     * @param array $selected_arr
     * @since [v1.0]
     * @return Array
     */
    public static function selectedPermissionsArray($permissions, $selected_arr = array())
    {
    	// return $selected_arr;
        $permissions_arr = array();

        foreach ($permissions as $permission) 
        {
            for ($x = 0; $x < count($permission); $x++) 
            {
                $permission_name = $permission[$x]['permission'];

                if ($permission[$x]['display'] === true) 
                {
                    if ($selected_arr) 
                    {
                        if (array_key_exists($permission_name,$selected_arr)) 
                        {

                            $permissions_arr[$permission_name] = $selected_arr[$permission_name];
                        } 
                        else 
                        {
                        	$permissions_arr[$permission_name] = '-1';
                        }
                    } 
                    else 
                    {
                        $permissions_arr[$permission_name] = '-1';
                    }
                }


            }


        }

        return $permissions_arr;
    }

    /**
     * Introspects into the model validation to see if the field passed is required.
     * This is used by the blades to add a required class onto the HTML element.
     * This isn't critical, but is helpful to keep form fields in sync with the actual
     * model level validation.
     *
     * This does not currently handle form request validation requiredness :(
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v3.0]
     * @return boolean
     */
    public static function checkIfRequired($class, $field) {
        $rules = $class::rules();
        foreach ($rules as $rule_name => $rule) {
            if ($rule_name == $field) {
                if (strpos($rule, 'required') === false) {
                    return false;
                } else {
                    return true;
                }

            }
        }
    }


    /**
     * Check to see if the given key exists in the array, and trim excess white space before returning it
     *
     * @author Daniel Melzter
     * @since 3.0
     * @param $array array
     * @param $key string
     * @param $default string
     * @return string
     */
    public static function array_smart_fetch(array $array, $key, $default = '')
    {
       array_change_key_case($array, CASE_LOWER);
        return array_key_exists(strtolower($key), array_change_key_case($array)) ? e(trim($array[ $key ])) : $default;
    }

    /**
     * Check to see if the given key exists in the array, and trim excess white space before returning it
     *
     * @author A. Gianotto
     * @since 3.2
     * @param $array array
     * @return string
     */
    public static function getLastDateFromHistoryArray(array $array)
    {
        foreach ($array as $key => $value) {
//            echo '<pre>';
//            echo 'last:'.$key;
//            print_r($array);
//            echo '</pre>';
        }
    }


    /**
     * Gracefully handle decrypting the legacy data (encrypted via mcrypt) and use the new
     * decryption method instead.
     *
     * This is not currently used, but will be.
     *
     * @author A. Gianotto
     * @since 3.6
     * @param CustomField $field
     * @param String $string
     * @return string
     */
    public static function gracefulDecrypt(CustomField $field, $string) {

        if ($field->isFieldDecryptable($string)) {

            try {
                Crypt::decrypt($string);
                return Crypt::decrypt($string);

            } catch (DecryptException $e) {
                return 'Error Decrypting: '.$e->getMessage();
            }

        }
        return $string;

    }

    /**
     * Strip HTML out of returned JSON. This is pretty gross, and I'd like to find a better way
     * to handle this, but the REST API will solve some of these problems anyway.
     *
     * This is not currently used, but will be.
     *
     * @author A. Gianotto
     * @since 3.4
     * @param $array array
     * @return Array
     */
    public static function stripTagsFromJSON(Array $array) {

        foreach ($array as $key => $value) {
            $clean_value = strip_tags($value);
            $clean_array[$key] = $clean_value;
        }
        return $clean_array;

    }

    /*
            Date: 17-04-2017
    */
            /**
     * Get the list of companies in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function clientList()
    {
       
        $company_list = array('' => trans('general.select_client')) + DB::table('companies')->where('status','=','Active')
                ->whereNull('deleted_at')
                ->orderBy('name', 'asc')
                ->pluck('name', 'id');
        return $company_list;
    }
    public static function customerList()
    {
       
        $company_list = array('' => trans('general.select_customer')) + DB::table('companies')
                ->where('status','=','Active')
                ->whereNull('deleted_at')
                ->orderBy('name', 'asc')
                ->pluck('name', 'id');
        return $company_list;
    }
     /**
     * Get the list of project in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function projectList($clientId = null)
    {
       $project_list =   array( '' => trans('general.select_project')) +
                        
             DB::table('project_detail')->where('companies_id','=',$clientId)->orderBy('project_name', 'asc')->pluck("project_name", 'id');
            
            return $project_list;
    }

    public static function AllprojectList($type = null)
    {
        $project_list =   array( '' => trans('general.select_project')) + DB::table('project_detail')
                        ->orderBy('project_name', 'asc')->pluck("project_name", 'id');
            
            return $project_list;
    }

    public static function serviceProjectList()
    {   
        $project_list =  DB::table('project_detail')->where("status","=","Active")->where('project_type', 'service')->orderBy('project_name', 'asc')->pluck('project_name', 'id');
        $project_list->prepend(trans('general.select_project'), '');
            
            return $project_list;
    } 
    public static function LMProjectList()
    {   $project_ids = Auth::user()->project_id;
        $project_ids = explode(',', $project_ids);
        $project_list=array();
        if(Auth::user()->user_role =='Logistic Manager'){
        $project_list =  DB::table('project_detail')->whereIN("project_detail.id",$project_ids)->orderBy('project_name', 'asc')->pluck('project_name', 'id');
        $project_list->prepend(trans('general.select_project'), '');
       }else{
          $project_list =  DB::table('project_detail')->orderBy('project_name', 'asc')->pluck('project_name', 'id');
        $project_list->prepend(trans('general.select_project'), '');
       }
            
            return $project_list;
    }

     /**
     * Get the list of project user in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function projectUserList()
    {
        $projectUser_list = array('' => trans('general.select_user')) +
                        User::where('deleted_at', '=', null)->where('blacklist','!=',1)
                        ->where('status','=','Active')
                        ->orderBy('last_name', 'asc')
                        ->orderBy('first_name', 'asc')->get()
                        ->pluck('full_name', 'id')->toArray();

        return $projectUser_list;
    }

    public static function projectttlActiveUserList()
    {
        $projectUser_list1 = User::where('deleted_at', '=', null)->where('blacklist','!=',1)
                        ->where('status','=','Active')
                        ->orderBy('last_name', 'asc')
                        ->orderBy('first_name', 'asc')->get()
                        ->pluck('full_name', 'id')->toArray();

        return $projectUser_list1;
    }
    /**
     * Get the list of locations in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */

    public static function multilocationsList()
    {
        $location_list = array('' => trans('general.select_site')) +Location::whereNull('deleted_at')
                ->where('status','=','Active')->orderBy('name', 'asc')
                ->pluck('name', 'id')->toArray();
        return $location_list;
    }
    public static function makeList()
    {
        $make_list = array('' => trans('general.select_make')) + WarrantyAsset::orderBy('name', 'asc')
                ->pluck('name', 'id')->toArray();
        return $make_list;
    }

    /**
     * Get the list of locations in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function projlocationsList($projectID = null)

    {
        $location_list = array('' => trans('general.select_location')) + 
        DB::table("project_detail")
            ->join("locations",DB::raw("FIND_IN_SET(locations.id,project_detail.location_id)"),">",DB::raw("'0'"))
            ->where("project_detail.id","=",$projectID)
            ->pluck('locations.name','locations.id');

        return $location_list;
    }
    /**
     * Get the list of managers in an array to make a dropdown menu
     *
     * @author [A. Gianotto] [<snipe@snipe.net>]
     * @since [v2.5]
     * @return Array
     */
    public static function employeeList()
    {
        $manager_list = array('' => trans('general.select_employee')) +
                        User::where('deleted_at', '=', null)->where('blacklist','!=',1)
                         ->where('status','=','Active')
                        ->orderBy('last_name', 'asc')
                        ->orderBy('first_name', 'asc')->get()
                        ->pluck('full_name', 'id')->toArray();

        return $manager_list;
    }
    /*
        Date : 20-04-2017

        get warranty models
    */
    public static function warrantymodelList()
    {
        $models = WarrantyAssetModel::with('manufacturer')->get();
        $model_array[''] = trans('general.select_model');
        foreach ($models as $model) {
            $model_array[$model->id] = $model->displayModelName();
        }
        return $model_array;
    }

    public static function assetTypeList()
    {
        $categories = Category::orderBy('name', 'asc')
                ->whereNull('deleted_at')
                ->orderBy('name', 'asc');

        $category_list = array('' => trans('general.select_category')) + $categories->pluck('name', 'id')->toArray();
      
        return $category_list;
    }

    public static function encryptor($action, $string)
    {
        $output = false;

        $encrypt_method = "AES-256-CBC";
        //pls set your unique hashing key
        $secret_key = 'parex';
        $secret_iv = 'parex123';

        // hash
        $key = hash('sha256', $secret_key);

        // iv - encrypt method AES-256-CBC expects 16 bytes - else you will get a warning
        $iv = substr(hash('sha256', $secret_iv), 0, 16);

        //do the encyption given text/string/number
        if( $action == 'encrypt' ) {
            $output = openssl_encrypt($string, $encrypt_method, $key, 0, $iv);
            $output = base64_encode($output);
        }
        else if( $action == 'decrypt' ){
            //decrypt the given text/string/number
            $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        }

        return $output;
    }

    public static function clientContactList($clientId = null)
    {
        $clientContactList = DB::table('companies_contact')->whereNull('deleted_at')->where('companies_id','=',$clientId)->pluck('name', 'id');
            
        return $clientContactList;
    }

    public static function supplierContactList($supplierId = null)
    {
        $supplierContactList = DB::table('suppliers_contact')->whereNull('deleted_at')->where('suppliers_id','=',$supplierId)->pluck('name', 'id');
            
        return $supplierContactList;
    }

    public static function skillList()
    {       
        $skill_list = array('' => trans('admin/users/general.select_skill')) + DB::table('skill')->orderBy('skill_name', 'asc')
                    ->pluck('skill_name', 'id');
        return $skill_list;
    }
    

    public static function deptList()
    {
        $dept_list = 
        // array('0' => trans('admin/users/general.select_dept')) + 
        DB::table('department')
                ->orderBy('dept_name', 'asc')
                ->pluck('dept_name', 'id');
        return $dept_list;
    }

    public static function domainList()
    {
        $domain_list = 
        // array('0' => trans('admin/users/general.select_domain')) +
         DB::table('domain')
                ->orderBy('domain_name', 'asc')
                ->pluck('domain_name', 'id');
        return $domain_list;
    }

    public static function pr($data,$ext=0)
    {
        echo "<pre>";
        print_r($data);
        if($ext == 1)
            exit;
        
    }

    public static function getDocumentType()
    {
        $document_type = array('' => 'select document') + DB::table('document_type_master')->where('status','=','Active')->pluck('name', 'id')->toArray();
        return $document_type;
    }
    public static function sendMail($to,$toname,$mailfrom,$mailfromname,$subject,$html,$text,$tag,$replyto,$cc=""){
        $array_data = array(
            'from'=> $mailfromname .'<'. $mailfrom .'>',
            'to'=> $toname .'<'. $to .'>', 
            'subject'=>$subject,
            'html'=>$html,
            'text'=>$text,
            'o:tracking'=>'yes',
            'o:tracking-clicks'=>'yes',
            'o:tracking-opens'=>'yes',
            'o:tag'=>$tag,
            'h:Reply-To'=>$replyto
        );
        !empty($cc)?$array_data['bcc'] = $cc:'';
        $session = curl_init(config('app.MAILGUN_URL').'/messages');
        curl_setopt($session, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($session, CURLOPT_USERPWD, 'api:'.config('app.MAILGUN_KEY'));
        curl_setopt($session, CURLOPT_POST, true);
        curl_setopt($session, CURLOPT_POSTFIELDS, $array_data);
        curl_setopt($session, CURLOPT_HEADER, false);
        curl_setopt($session, CURLOPT_ENCODING, 'UTF-8');
        curl_setopt($session, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($session, CURLOPT_SSL_VERIFYPEER, false);
        $response = curl_exec($session);
        curl_close($session);
        $results = json_decode($response, true);
        return $results;
    }
     public static function MarketplaseList($seller_id = null)
    {
        $MarketplaseList = DB::table('seller_marketplace_id')->whereNull('deleted_at')->where('seller_id','=',$seller_id)->pluck('az_marketplace_id')->toArray();
            
        return $MarketplaseList;
    }
    //Edited by rezker (http://www.rezker.com)
    public static function code_to_country( $code ){

    $code = strtoupper($code);

    $countryList = array('AF' => 'Afghanistan', 'AX' => 'Aland Islands', 'AL' => 'Albania', 'DZ' => 'Algeria', 'AS' => 'American Samoa', 'AD' => 'Andorra', 'AO' => 'Angola', 'AI' => 'Anguilla', 'AQ' => 'Antarctica', 'AG' => 'Antigua and Barbuda', 'AR' => 'Argentina', 'AM' => 'Armenia', 'AW' => 'Aruba', 'AU' => 'Australia', 'AT' => 'Austria', 'AZ' => 'Azerbaijan', 'BS' => 'Bahamas the', 'BH' => 'Bahrain', 'BD' => 'Bangladesh', 'BB' => 'Barbados', 'BY' => 'Belarus', 'BE' => 'Belgium', 'BZ' => 'Belize', 'BJ' => 'Benin', 'BM' => 'Bermuda', 'BT' => 'Bhutan', 'BO' => 'Bolivia', 'BA' => 'Bosnia and Herzegovina', 'BW' => 'Botswana', 'BV' => 'Bouvet Island (Bouvetoya)', 'BR' => 'Brazil', 'IO' => 'British Indian Ocean Territory (Chagos Archipelago)', 'VG' => 'British Virgin Islands', 'BN' => 'Brunei Darussalam', 'BG' => 'Bulgaria', 'BF' => 'Burkina Faso', 'BI' => 'Burundi', 'KH' => 'Cambodia', 'CM' => 'Cameroon', 'CA' => 'Canada', 'CV' => 'Cape Verde', 'KY' => 'Cayman Islands', 'CF' => 'Central African Republic', 'TD' => 'Chad', 'CL' => 'Chile', 'CN' => 'China', 'CX' => 'Christmas Island', 'CC' => 'Cocos (Keeling) Islands', 'CO' => 'Colombia', 'KM' => 'Comoros the', 'CD' => 'Congo', 'CG' => 'Congo the', 'CK' => 'Cook Islands', 'CR' => 'Costa Rica', 'CI' => 'Cote d\'Ivoire', 'HR' => 'Croatia', 'CU' => 'Cuba', 'CY' => 'Cyprus', 'CZ' => 'Czech Republic', 'DK' => 'Denmark', 'DJ' => 'Djibouti', 'DM' => 'Dominica', 'DO' => 'Dominican Republic', 'EC' => 'Ecuador', 'EG' => 'Egypt', 'SV' => 'El Salvador', 'GQ' => 'Equatorial Guinea', 'ER' => 'Eritrea', 'EE' => 'Estonia', 'ET' => 'Ethiopia', 'FO' => 'Faroe Islands', 'FK' => 'Falkland Islands (Malvinas)', 'FJ' => 'Fiji the Fiji Islands', 'FI' => 'Finland', 'FR' => 'France, French Republic', 'GF' => 'French Guiana', 'PF' => 'French Polynesia', 'TF' => 'French Southern Territories', 'GA' => 'Gabon', 'GM' => 'Gambia the', 'GE' => 'Georgia', 'DE' => 'Germany', 'GH' => 'Ghana', 'GI' => 'Gibraltar', 'GR' => 'Greece', 'GL' => 'Greenland', 'GD' => 'Grenada', 'GP' => 'Guadeloupe', 'GU' => 'Guam', 'GT' => 'Guatemala', 'GG' => 'Guernsey', 'GN' => 'Guinea', 'GW' => 'Guinea-Bissau', 'GY' => 'Guyana', 'HT' => 'Haiti', 'HM' => 'Heard Island and McDonald Islands', 'VA' => 'Holy See (Vatican City State)', 'HN' => 'Honduras', 'HK' => 'Hong Kong', 'HU' => 'Hungary', 'IS' => 'Iceland', 'IN' => 'India', 'ID' => 'Indonesia', 'IR' => 'Iran', 'IQ' => 'Iraq', 'IE' => 'Ireland', 'IM' => 'Isle of Man', 'IL' => 'Israel', 'IT' => 'Italy', 'JM' => 'Jamaica', 'JP' => 'Japan', 'JE' => 'Jersey', 'JO' => 'Jordan', 'KZ' => 'Kazakhstan', 'KE' => 'Kenya', 'KI' => 'Kiribati', 'KP' => 'Korea', 'KR' => 'Korea', 'KW' => 'Kuwait', 'KG' => 'Kyrgyz Republic', 'LA' => 'Lao', 'LV' => 'Latvia', 'LB' => 'Lebanon', 'LS' => 'Lesotho', 'LR' => 'Liberia', 'LY' => 'Libyan Arab Jamahiriya', 'LI' => 'Liechtenstein', 'LT' => 'Lithuania', 'LU' => 'Luxembourg', 'MO' => 'Macao', 'MK' => 'Macedonia', 'MG' => 'Madagascar', 'MW' => 'Malawi', 'MY' => 'Malaysia', 'MV' => 'Maldives', 'ML' => 'Mali', 'MT' => 'Malta', 'MH' => 'Marshall Islands', 'MQ' => 'Martinique', 'MR' => 'Mauritania', 'MU' => 'Mauritius', 'YT' => 'Mayotte', 'MX' => 'Mexico', 'FM' => 'Micronesia', 'MD' => 'Moldova', 'MC' => 'Monaco', 'MN' => 'Mongolia', 'ME' => 'Montenegro', 'MS' => 'Montserrat', 'MA' => 'Morocco', 'MZ' => 'Mozambique', 'MM' => 'Myanmar', 'NA' => 'Namibia', 'NR' => 'Nauru', 'NP' => 'Nepal', 'AN' => 'Netherlands Antilles', 'NL' => 'Netherlands the', 'NC' => 'New Caledonia', 'NZ' => 'New Zealand', 'NI' => 'Nicaragua', 'NE' => 'Niger', 'NG' => 'Nigeria', 'NU' => 'Niue', 'NF' => 'Norfolk Island', 'MP' => 'Northern Mariana Islands', 'NO' => 'Norway', 'OM' => 'Oman', 'PK' => 'Pakistan', 'PW' => 'Palau', 'PS' => 'Palestinian Territory', 'PA' => 'Panama', 'PG' => 'Papua New Guinea', 'PY' => 'Paraguay', 'PE' => 'Peru', 'PH' => 'Philippines', 'PN' => 'Pitcairn Islands', 'PL' => 'Poland', 'PT' => 'Portugal, Portuguese Republic', 'PR' => 'Puerto Rico', 'QA' => 'Qatar', 'RE' => 'Reunion', 'RO' => 'Romania', 'RU' => 'Russian Federation', 'RW' => 'Rwanda', 'BL' => 'Saint Barthelemy', 'SH' => 'Saint Helena', 'KN' => 'Saint Kitts and Nevis', 'LC' => 'Saint Lucia', 'MF' => 'Saint Martin', 'PM' => 'Saint Pierre and Miquelon', 'VC' => 'Saint Vincent and the Grenadines', 'WS' => 'Samoa', 'SM' => 'San Marino', 'ST' => 'Sao Tome and Principe', 'SA' => 'Saudi Arabia', 'SN' => 'Senegal', 'RS' => 'Serbia', 'SC' => 'Seychelles', 'SL' => 'Sierra Leone', 'SG' => 'Singapore', 'SK' => 'Slovakia (Slovak Republic)', 'SI' => 'Slovenia', 'SB' => 'Solomon Islands', 'SO' => 'Somalia, Somali Republic', 'ZA' => 'South Africa', 'GS' => 'South Georgia and the South Sandwich Islands', 'ES' => 'Spain', 'LK' => 'Sri Lanka', 'SD' => 'Sudan', 'SR' => 'Suriname', 'SJ' => 'Svalbard & Jan Mayen Islands', 'SZ' => 'Swaziland', 'SE' => 'Sweden', 'CH' => 'Switzerland, Swiss Confederation', 'SY' => 'Syrian Arab Republic', 'TW' => 'Taiwan', 'TJ' => 'Tajikistan', 'TZ' => 'Tanzania', 'TH' => 'Thailand', 'TL' => 'Timor-Leste', 'TG' => 'Togo', 'TK' => 'Tokelau', 'TO' => 'Tonga', 'TT' => 'Trinidad and Tobago', 'TN' => 'Tunisia', 'TR' => 'Turkey', 'TM' => 'Turkmenistan', 'TC' => 'Turks and Caicos Islands', 'TV' => 'Tuvalu', 'UG' => 'Uganda', 'UA' => 'Ukraine', 'AE' => 'United Arab Emirates', 'GB' => 'United Kingdom', 'US' => 'United States of America', 'UM' => 'United States Minor Outlying Islands', 'VI' => 'United States Virgin Islands', 'UY' => 'Uruguay, Eastern Republic of', 'UZ' => 'Uzbekistan', 'VU' => 'Vanuatu', 'VE' => 'Venezuela', 'VN' => 'Vietnam', 'WF' => 'Wallis and Futuna', 'EH' => 'Western Sahara', 'YE' => 'Yemen', 'ZM' => 'Zambia', 'ZW' => 'Zimbabwe');
    if( !$countryList[$code] ) return $code;
    else return $countryList[$code];
    }

    public static function getModuleName($moduleName){
        if($moduleName == 'tlaz'){
            return config('app.moduleNames')['tlaz'];
        }else{
            return $moduleName;
        }
    }

    public static function trimInvalidCharacter($string){

        $string = strip_tags($string);
        return str_replace(array('<', '>', '&#39;'), array('', '', "'"), $string);
    }

    public static function isJSON($string)
    {
        return is_string($string) && is_array(json_decode($string, true)) && (json_last_error() == JSON_ERROR_NONE) ? true : false;
    }

    public static function array_change_key_case_recursive($arr, $case = CASE_LOWER)
    {
        return array_map(function ($item) use ($case) {
            if (is_array($item))
                $item = self::array_change_key_case_recursive($item, $case);
            return $item;
        }, array_change_key_case($arr, $case));
    }

    //Below function convert all the json inputs to array
    public static function convertJsonToArray($arrayData){
        foreach ($arrayData as $key => $value) {
            if ((strpos($value, '[', 0) !== false || strpos($value, '{', 0) !== false) && !empty($value)) {
                $arrayData[$key] = json_decode($value, true);
            } else {
                $arrayData[$key] = $value;
            }
        }
        return $arrayData;
    }

    /*
    Sample valid Email addresses
    [0] => brendan.gilbert+basandlokes@gmail.com
    [1] => test@domain.com
    [2] => lastname@domain.com
    [3] => test.email.with+symbol@domain.com
    [4] => id-with-dash@domain.com
    [5] => "xyz.test.@.test.com"@domain.com
    [6] => "abc.(),:;<>[]\".EMAIL.\"email@\ \"email\".test"@strange.domain.com
    [7] => example-abc@abc-domain.com
    [8] => #!$%&'*+-/=?^_{}|~@domain.org
    [9] => example@s.solutions
    [10] => test@[IPv6:2018:db8::1]
    */
    public static function valid_email($email) //To check email before send it to Tally to avoid errors from Tally
    {
        return (!filter_var($email, FILTER_VALIDATE_EMAIL)) ? FALSE : TRUE;
    }

    public static function trimAsciiCharacter($string)
    {
        //$string = iconv('UTF-8', 'ASCII//TRANSLIT', $string);
        $string = preg_replace_callback('/\\\\u([0-9a-fA-F]{4})/', function ($match) {
            return mb_convert_encoding(pack('H*', $match[1]), 'UTF-8', 'UTF-16BE');
        }, $string);
        $string = preg_replace('/[\x00-\x1F\x7F\xA0]/u', '', $string);
        
        return $string;
    }
}
