<?php

namespace App\Http\Controllers\Common;

use App\Helpers\Helper;
use App\Http\Controllers\Controller;
use App\Models\Accounts as AccountsDB;
use Illuminate\Http\Request;
use App\Models\TallyConnection;
use App\Http\Traits\Accounts;
use App\Http\Traits\Tax;
use App\Http\Traits\Requestresponse;
use Log;

class AccountsController extends Controller
{
    use Accounts;
    use Tax;
    use Requestresponse;
    public function getAccounts(Request $request)
    {
        $data=[];
        $seller_id = $request->seller_id;
        $connectionDetails = TallyConnection::checkTallyConnection($seller_id, $request->module);
        if(!empty($connectionDetails)){
            $ident = "LedgerList-".$seller_id."-".$connectionDetails->id;
            list($reqDataAry,$extraData) = $this->getAccountsXml($connectionDetails);
            $response= $this->tallyApi($connectionDetails,$ident,"LedgerList",$reqDataAry,$extraData);
            $data = $this->handleResponse($response);
        }else{
            $data['success'] = false;
            $data['message'] = "Tally not connected";
            $code= 401;
        }
        return $data;
       
    }

    public function createAccounts(Request $request)
    {
        $validated = $request->validate([
            'seller_id' => 'required',
            'accounts.*.name' => 'required',
            'accounts.*.parent' => 'required',
            'module'=> 'required',
        ]);

        $data=$reqAry=[];
        $seller_id = $request->seller_id;
        $accountsData = $request['accounts'];
        foreach($accountsData as $accountName){
            $names[] = $accountName['name'];
        }
        
        /*$accountAlreadyExist = AccountsDB::whereIn('name',$names)->where('seller_id',$seller_id)->where('module', $request->module)->pluck('name')->toArray();
        if(!empty($accountAlreadyExist)){
            $data['success'] = false;
            $data['data'] = $accountAlreadyExist;
            $data['message'] = "Queue already exists with same action";
            return $data;
        }*/
        
        $connectionDetails = TallyConnection::checkTallyConnection($seller_id, $request->module);
        if(!empty($connectionDetails)){
            foreach($accountsData as $key => $account){
                $actionType = "LedgerCreate";
                if($account['parent'] == "Sundry Debtors"){
                    $actionType = "SalesCustomerCreate";
                } else if($account['parent'] == "Sundry Creditors"){
                    $actionType = "PurchaseCustomerCreate";
                }
                $ident = $actionType."-".$seller_id."-".$connectionDetails->id."-".$key;
                list($ledgerArray,$extraData) = $this->createAccountXml($account,$connectionDetails);
                $reqDataAry = $this->generateRequestArray($account, $connectionDetails, $ledgerArray);
                $response = $this->tallyApi($connectionDetails,$ident,$actionType,$reqDataAry,$extraData);
                $data = $this->handleResponse($response);
            }
        }else{
            $data['success'] = false;
            $data['message'] = "Tally not connected for this store";
            $code= 401;
        }
        
        return $data;
       
    }

    public function createTaxAccount(Request $request)
    {
        $validated = $request->validate([
            'seller_id' => 'required',
            'accounts.*.name' => 'required',
            'accounts.*.parent' => 'required',
            'accounts.*.gst_duty' => 'required',
            'accounts.*.type_of_supply' => 'required',
            'accounts.*.round_method' => 'required',
            // 'accounts.*.category' => 'required',
            'accounts.*.rate_of_tax_calculation' => 'required',
            'module'=> 'required',
        ]);

        $data=$reqAry=[];
        $seller_id = $request->seller_id;
        $accountsData = $request['accounts'];
        foreach($accountsData as $accountName){
            $names[] = $accountName['name'];
        }
        
        $accountAlreadyExist = AccountsDB::whereIn('name',$names)->where('seller_id',$seller_id)->where('module', $request->module)->pluck('name')->toArray();
        if(!empty($accountAlreadyExist)){
            $data['success'] = false;
            $data['data'] = $accountAlreadyExist;
            $data['message'] = "Queue already exists with same action";
            return $data;
        }
        
        $connectionDetails = TallyConnection::checkTallyConnection($seller_id, $request->module);
        if(!empty($connectionDetails)){
            
            foreach($accountsData as $key => $account){
                $ident="TaxCreate-".$key;
                list($reqDataAry,$extraData) = $this->createTaxAccountXml($connectionDetails);
                $reqDataAry = $this->generateRequestArray($account, $connectionDetails, $ledgerArray);
                $response = $this->tallyApi($connectionDetails,$ident,"TaxCreate",$reqDataAry,$extraData);
                $data = $this->handleResponse($response);
            }

        }else{
            $data['success'] = false;
            $data['message'] = "Tally not connected for this store";
            $code= 401;
        }
        
        return $data;
       
    }

    
}
