<?php

namespace App\Http\Controllers\Common;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use App\Helpers\Helper;
use DateTime;
use Log;

class CustomizefeatureController extends Controller
{
    public function getItemsWithAccounts($tallySyncSetting, $taxKey, $itemKey, $parentKey, $orderData) {	
        
        $itemData = array('accountCode' => '', 'taxRate'=> '', 'description'=>'');
        //Find key based on field and parent field
        $item = $this->getPayoutField($tallySyncSetting['payout_field_mapping'], $itemKey, $parentKey);

        //Find account and tax details based on key from summary_sync_settings
        foreach($tallySyncSetting['sync_settings']['account_settings'] as $key=>$value){
            if(array_key_exists($item['field'] . '_acc',$value)){
                $taxCode = '';
                if(!empty($value[$item['field'] . '_tax_rate'])){
                    $taxCode = $value[$item['field'] . '_tax_rate'];    
                } else if (!empty($value[$item['field'] . '_tax_code'])) {
                    $taxCode = $value[$item['field'] . '_tax_code'];
                }
                $this->setAccountTaxCode($itemData, $value[$item['field'] . '_acc'], $taxCode, $item['description']);
                break;
            } else if(array_key_exists('other_acc', $value)){
                $taxCode = '';
                if (!empty($value['other_tax_rate'])) {
                    $taxCode = $value['other_tax_rate'];
                } else if (!empty($value['other_tax_code'])) {
                    $taxCode = $value['other_tax_code'];
                }
                $this->setAccountTaxCode($itemData, $value['other_acc'], $taxCode, $item['description']);
            }
        }

        if(!empty($tallySyncSetting['sync_settings']['multi_tax_code']) && array_key_exists($taxKey, $tallySyncSetting['sync_settings']['multi_tax_code']) && !empty($tallySyncSetting['sync_settings']['multi_tax_code'][$taxKey])){
            $this->setAccountTaxCode($itemData, '', $tallySyncSetting['sync_settings']['multi_tax_code'][$taxKey], '');
        }
        
        //Find accounts based on summary_account_mapping without country
        if(!empty($tallySyncSetting['account_mapping']['mapping'])){
            $itemData = $this->getAccountMapping($tallySyncSetting['account_mapping']['mapping'], $parentKey, $itemKey, $taxKey, $itemData);
        }

        //Find account and tax details based on key matched in summary_sync_settings_x country_code
        if (!empty($tallySyncSetting['country_sync_settings_array']['account_settings'])) {
            foreach ($tallySyncSetting['country_sync_settings_array']['account_settings'] as $key => $value) {
                if (array_key_exists($item['field'] . '_acc', $value)) {
                    $taxCode = '';
                    if (!empty($value[$item['field'] . '_tax_rate'])) {
                        $taxCode = $value[$item['field'] . '_tax_rate'];
                    } else if (!empty($value[$item['field'] . '_tax_code'])) {
                        $taxCode = $value[$item['field'] . '_tax_code'];
                    }
                    $this->setAccountTaxCode($itemData, $value[$item['field'] . '_acc'], $taxCode, $item['description']);
                    break;
                }
            }
        }
        
        //Find accounts based on summary_account_mapping with country matched
        if (!empty($tallySyncSetting['country_account_mapping_array']['mapping'])) {
            $itemData = $this->getAccountMapping($tallySyncSetting['country_account_mapping_array']['mapping'], $parentKey, $itemKey, $taxKey, $itemData);
        }

        if (!empty($tallySyncSetting['sync_settings']['must_apply_tax']) && $tallySyncSetting['sync_settings']['must_apply_tax'] == 'Yes' && !empty($tallySyncSetting['sync_settings']['multi_tax_code']['must_tax']) && in_array(strtolower($itemKey), array_map('strtolower', config('mastertallyconstant.' . $tallySyncSetting['sync_settings']['module'] . '.APPLYTAXMUSTKEYS')))) {
            $this->setAccountTaxCode($itemData, '', $tallySyncSetting['sync_settings']['multi_tax_code']['must_tax'], '');
        }
        
        //Find accounts based on review
        $draftData = json_decode($orderData['draft_data'], true);
        $itemData = $this->getAccountMapping($draftData, $parentKey, $itemKey, $taxKey, $itemData);
        
        //If we are syncing fees of summary then summary_id will not be blank and we will use that account for create bill. generally it will be for xoaz, xoebay, xoetsy. NS:10-01-2023
        // if(!empty($orderData['xero_summary_id'])){
            
        //     $accountMappingData = Xerosummaryaccmappinglog::getSummaryMappingData($tallySyncSetting['sync_settings'], $orderData['xero_summary_id']);
        //     $itemData = $this->getAccountMapping($accountMappingData, $parentKey, $itemKey, $taxKey, $itemData);
        // }
        
        return $itemData;
    }

    

    public function getPayoutField($payoutFieldMapping,$item, $parent){
        //Default Account and Description
		
        $fieldData = array('field'=>'other', 'description'=>'Other');
        $isParentFieldFlag = false;

        foreach($payoutFieldMapping as $key=>$value){
			
            if(strtolower($item) == strtolower($value['field']) && strtolower($parent) == strtolower($value['parent_field'])){
                //If field and parent_field matched
                $fieldData = array('field' => $value['key'], 'description' => !empty($value['description']) ? $value['description'] : '');
                $isParentFieldFlag = true;
                break;
            }
        }
		

        if($isParentFieldFlag == false){
            foreach ($payoutFieldMapping as $key => $value) {
                if (strtolower($item) == strtolower($value['field']) && empty($value['parent_field'])) {
                    //If field match with parent_field = empty
                    $fieldData = array('field' => $value['key'], 'description' => !empty($value['description']) ? $value['description'] : '');
                    break;
                }
            }
        }

        return $fieldData;
    }

    protected function setAccountTaxCode(&$itemData, $accountCode, $taxCode, $description){
        if(!empty($accountCode))
            $itemData['accountCode'] = $accountCode;
        if (!empty($taxCode))
            $itemData['taxRate'] = $taxCode;
        if (!empty($description))
            $itemData['description'] = $description;
    }

    public function getAccountMapping($mapping, $parentKey, $itemKey, $taxKey, $itemData){
        if(!empty($mapping)){
            if (array_key_exists($parentKey, $mapping) && !empty($mapping[$parentKey]['account'])) {
                $data = $mapping[$parentKey];
                $itemData = array(
                    'accountCode' => $data['account'], 
                    'taxRate' => !empty($data['taxrate']) ? $data['taxrate'] : $itemData['taxRate'], 
                    'description' => !empty($data['description']) ? $data['description'] : $itemData['description']
                );
            }

            //If match then overwrite above
            if (!empty($mapping[$parentKey]) && array_key_exists($itemKey, $mapping[$parentKey]) && !empty($mapping[$parentKey][$itemKey]['account'])) {
                $data = $mapping[$parentKey][$itemKey];
                $itemData = array(
                    'accountCode' => $data['account'], 
                    'taxRate' => !empty($data['taxrate']) ? $data['taxrate'] : $itemData['taxRate'], 
                    'description' => !empty($data['description']) ? $data['description'] : $itemData['description']
                );
            }

            //If match then overwrite above
            if (!empty($mapping[$parentKey][$itemKey]) && array_key_exists($taxKey, $mapping[$parentKey][$itemKey]) && !empty($mapping[$parentKey][$itemKey][$taxKey]['account'])) {
                $data = $mapping[$parentKey][$itemKey][$taxKey];
                $itemData = array(
                    'accountCode' => $data['account'], 
                    'taxRate' => !empty($data['taxrate']) ? $data['taxrate'] : $itemData['taxRate'], 
                    'description' => !empty($data['description']) ? $data['description'] : $itemData['description']
                );
            }
        }
        
        return $itemData;
    }

    public function setSummaryAccountMapping($tallyConnected, $orderData = array(), $superParent, $parent, $child)
    {
        $accrefCode = '';
        if (array_key_exists("OACCMAPPING", $tallyConnected['customConfig']['featureList']) && !empty($orderData)) {
            $featureData = json_decode($tallyConnected['customConfig']['featureList']['OACCMAPPING'], true);

            if(!empty($featureData))
            {
                $data = $featureData;
                $oAccOrderData = array_dot($orderData);
                foreach ($data as $key => $value) {
                    if (array_key_exists($key, $oAccOrderData) && !empty($data[$key][$oAccOrderData[$key]])) {
                        $superParentKey = $superParent;
                        $parentKey = $superParent.'.'.$parent;
                        $childKey = $superParent.'.'.$parent.'.'.$child;
                        if(array_key_exists($superParentKey,$data[$key][$oAccOrderData[$key]])){
                            $accrefCode = $data[$key][$oAccOrderData[$key]][$superParentKey];
                        }
                        if(array_key_exists($parentKey,$data[$key][$oAccOrderData[$key]])){
                            $accrefCode = $data[$key][$oAccOrderData[$key]][$parentKey];
                        }
                        if(array_key_exists($childKey,$data[$key][$oAccOrderData[$key]])){
                            $accrefCode = $data[$key][$oAccOrderData[$key]][$childKey];
                        }
                        // $accrefCode = $data[$key][$oAccOrderData[$key]];
                    } 
                    // else if (!empty($data[$key]['other'])) {
                    //     $accrefCode = $data[$key]['other'];
                    // }
                }
            }
        }
        return $accrefCode;
    }

     /*
        DB table : tally_feature
        feature_code : INVTRACKCAT 
        FORMAT: 
        {"marketplace_record_type":{"payout":{"class":"1","hoby":"movie"}}}
        {"ALL":{"class":"2","hoby":"movie1"},"marketplace_record_type":{"payout":{"class":"1","hoby":"movie"}}}
    */
    public function getSummaryInvTreckingCat($orderData, $lineItemData, $tallyConnected)
    {
        $trackCategoryAllData = array();
        if (array_key_exists("INVTRACKCAT", $tallyConnected['customConfig']['featureList'])) {
            $featureDataArray = json_decode($tallyConnected['customConfig']['featureList']['INVTRACKCAT'], true);
           
            $orderData = array_dot($orderData);
            foreach ($featureDataArray as $key => $value) {
                if (strtolower($key) == strtolower( "ALL") && !empty($value)) {
                    $trackCategoryAllData = $value;
                }
                if(isset($orderData[$key])){
                    $orderData[$key] = !empty($orderData[$key]) ? $orderData[$key] : 'blank';
                    foreach ($value as $key2 => $value2) {
                        if (array_key_exists($key, $orderData)) {
                            if (isset($orderData[$key]) && strtolower($orderData[$key]) == strtolower($key2)) {
                                $trackCategoryAllData = $value2;
                                break; break;
                            }
                        }
                    }
                }
            }
        }
        return $trackCategoryAllData;
    }

    public function getSettlementAccounts($tallyConnected,$tallySyncSetting, $order)
    {
        $accountCode = '';
        if(!empty($tallySyncSetting['sync_settings']['bank_account'])){
            $accountCode = $tallySyncSetting['sync_settings']['bank_account'];
        }
        return $accountCode;

    }

}
