<?php

namespace App\Http\Controllers\Common;

use App\Models\Tallycompany;
use Illuminate\Http\Request;
use App\Models\TallyConnection;
use Illuminate\Support\Facades\Response;
use App\Services\Account\Account;
use App\Http\Controllers\Controller;
use App\Http\Traits\Requestresponse;
use App\Http\Traits\Company;
use App\Http\Traits\Journal;
use App\Http\Traits\Item;
use App\Http\Traits\Receipt;
use App\Http\Traits\Accounts;
use Log;

class TallylibraryController extends Controller
{
	/** 
	 * Display a listing of the resource.
	 *
	 * CREATE CONNECTION DYNAMIC tally_connections 
	 * aa api ekj var call thase jyare front side thi call kare chhe tyare seller_id module badha ma add karvana chhe
	 * @return \Illuminate\Http\Response
	 */
	use Requestresponse;
	use Company;
	use Journal;
	use Item;
	use Receipt;
	use Accounts;
	public function connect(Request $request)
	{
		Log::info('request comes here');
		//CHECK VALIDATION FOR REQUIRED FIELD
		$validated = $request->validate([
            'seller_id' => 'required',
            'username' => 'required',
            'password' => 'required',
			// 'userId'=> 'required',
			'module'=> 'required',
        ]);
		
		$data = [
			'username' => $request->username,
			'password' => $request->password,
			'companyName' => $request->companyName ? $request->companyName : '',
			'userId'   => 1,//$request->userId,
			'seller_id'   => $request->seller_id,
			'module' => $request->module
		];
		$response = $this->connectTally($data);		

		if ($response->success == 1) {
			$connectionId = (isset($response->data->tallyConnectionId)) ? $response->data->tallyConnectionId : "";
			TallyConnection::createUser($data, $connectionId);
			
			// FETCH COMPANY LIST QUEUE CREATE
			$connectionDetails=TallyConnection::checkTallyConnection($request->seller_id,$request->module);
			if(!empty($connectionDetails)){
				$ident="CompanyList-". $request->seller_id."-".$connectionDetails->id;
				list($reqDataAry,$extraData) = $this->getCompanyListXml($connectionDetails);
				$companyListResponse = $this->tallyApi($connectionDetails,$ident,"CompanyList",$reqDataAry,$extraData);

				if($companyListResponse->success){
					$updateData = ['company_list_flag' => 0];
					Tallycompany::updateCompanyData($request->seller_id, $request->module, $updateData);
					$responseData['success'] = true;
					$responseData['message'] = $companyListResponse->message;
				}else{
					$responseData['success'] = false;
					$responseData['message'] = $companyListResponse->message;
				}
			}else{
				$responseData['success'] = false;
				$responseData['message'] = "Tally not connected";
			}
			$responseData['success'] = true;
			$responseData['message']    = "Tally connected Successfully.";
			$responseData['tally_url'] = $response->data->tallyConfigUrl;
			$responseData['password'] = $response->data->password;
		} else {
			$responseData['success'] = false;
			$responseData['data']    = [];
			$responseData['message'] = $response->message;
		}
		return $responseData;
	}


	/** 
	 * Display a listing of the resource.
	 *
	 * REGENERATE tally_connections FILE 
	 * aa api front side thi call kare chhe tyare call thase connection file get karva mate je file avse response ma a return kari desu
	 * @return \Illuminate\Http\Response
	 */
	public function reGenerateFile(Request $request)
	{
		//CHECK VALIDATION FOR REQUIRED FIELD
		$validated = $request->validate([
            'seller_id' => 'required',
            'username' => 'required',
            'password' => 'required',
			'module'=> 'required',
        ]);
		
		$data = [
			'username' => $request->username,
			'password' => $request->password,
			'companyName' => $request->companyName ? $request->companyName : '',
			'seller_id'   => $request->seller_id,
			'module' => $request->module
		];

		$response = $this->regenerateTallyFile($data);
		if ($response->success == 1) {
			$connectionId = (isset($response->data->tallyConnectionId)) ? $response->data->tallyConnectionId : "";
			
			$responseData['success'] = true;
			$responseData['message']    = "Tally connected Successfully.";
			$responseData['tally_url'] = $response->data->tallyConfigUrl;
			$responseData['password'] = $response->data->password;
		} else {
			$responseData['success'] = false;
			$responseData['data']    = [];
			$responseData['message'] = $response->message;
		}
		return $responseData;
	}
	
	public function TallyRequest($ident, $type = "", $reqDataAry, $extraData)
	{
		$req = array();
		switch ($type) {
			case "LedgerCreate":
			break;
			case "TaxCreate":
			break;
			case "SalesCustomerCreate":
				$req=$this->xmlRequest('SalesCustomerCreate', "create", $ident, $reqDataAry, $extraData);
				break;
			break;
			case "PurchaseCustomerCreate":
			break;
			case "ItemCreate":
				$req=$this->xmlRequest($type, "create", $ident, $reqDataAry, $extraData);
				break;
			case "LedgerUpdate":
				$req=$this->xmlRequest("LedgerUpdate", "update", $ident, $reqDataAry, $extraData);
				break;
			case "LedgerList":
				$req=$this->xmlRequest("LedgerList", "create", $ident, $reqDataAry, $extraData);
				break;
			case "LedgerDelete":
				$req=$this->xmlRequest("LedgerDelete", "delete", $ident, $reqDataAry, $extraData);
				break;
			case "JournalCreate":
				$req=$this->xmlRequest("JournalCreate", "create", $ident, $reqDataAry, $extraData);
				break;
			case "CreditNoteCreate":
			break;
			case "SalesVoucherCreate":
			break;
			case "JournalUpdate":
			break;
			case "CreditNoteUpdate":
			break;
			case "SalesVoucherUpdate":
			break;
			case "SalesInvoiceCreate":
				$req=$this->xmlRequest($type,"create",$ident,$reqDataAry,$extraData);
				break;
			case "SalesInvoiceUpdate":
				$req=$this->xmlRequest($type,"create",$ident,$reqDataAry,$extraData);
				break;
			case "VoucherCollection":
				$req=$this->xmlRequest("VoucherCollection","create",$ident,$reqDataAry,$extraData);
				break;
			case "CompanyList":
				$req=$this->xmlRequest("CompanyList","create",$ident,$reqDataAry,$extraData);
				break;
			case "updateCompanyData":
				$req=$this->xmlRequest("updateCompanyData","create",$ident,$reqDataAry,$extraData);
				break;
			case "ItemCollection":
				$req=$this->xmlRequest("ItemCollection","create",$ident,$reqDataAry,$extraData);
				break;
			case "ReceiptCreate":
				$req=$this->xmlRequest("ReceiptCreate","create",$ident,$reqDataAry,$extraData);
				break;
		}
		return $req;
	}
	
	public function tallyResponse(Request $request)
	{

		Log::info('tally response');
		Log::info(print_r($request->all(),1));

		$action = $request['action'];

		switch ($action) {
			case "LedgerCreate":
			break;
			case "TaxCreate":
			break;
			case "SalesCustomerCreate":
				$this->LedgerCreateResponse($request->all());
				break;
			break;
			case "PurchaseCustomerCreate":
				$this->LedgerCreateResponse($request->all());
				break;
			case "LedgerList":
				$this->ledgerListResponse($request->all());
				break;
			case "JournalCreate":
			break;
			case "CreditNoteCreate":
			break;
			case "SalesVoucherCreate":
			break;
			case "JournalUpdate":
			break;
			case "CreditNoteUpdate":
			break;
			case "SalesVoucherUpdate":
				$this->JournalCreate($request->all());
				break;
			case "VoucherCollection":
				$this->VoucherCollection($request->all());
				break;
			case "CompanyList":
				$this->CompanyListResponse($request->all());
				break;
			case "updateCompanyData":
				$this->CompanyUpdateResponse($request->all());
				break;
			case "ItemCollection":
				$this->ItemsCollection($request->all());
				break;
			case "ItemCreate":
				$this->ItemsCreate($request->all());
				break;
			case "SalesInvoiceCreate":
				$this->invoiceCreate($request->all());
				break;
			case "SalesInvoiceUpdate":
				$this->invoiceCreate($request->all());
				break;
			case "ReceiptCreate":
				$this->receiptCreate($request->all());
				break;
		}
	}
}
