<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Models\Accounts;
use App\Models\Sellers;
use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Routing\Controller as BaseController;
use Illuminate\Support\Js;
use Illuminate\Http\Request;
use App\Models\TallyConnection;
use App\Models\Tallypayoutfieldmapping;
use App\Models\TallySettings;
use App\Models\Tallysummaryaccountmapping;
use App\Models\Tallysummarysettings;
use Illuminate\Support\Facades\Response;
use App\Services\Account\Account;
use Illuminate\Support\Facades\DB;
use Log;

class Controller extends BaseController
{
    use AuthorizesRequests, DispatchesJobs, ValidatesRequests;
    public static function connectTally($data)
    {
        $curlUrl = config('app.tally_url').'createconnection';
        
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $curlUrl,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => json_encode($data),
            CURLOPT_HTTPHEADER => array(
                'Content-Type: application/json'
            ),
            CURLOPT_SSL_VERIFYHOST=>0,
            CURLOPT_SSL_VERIFYPEER=>0
        ));
        // Send the request and get the response
        $response = curl_exec($curl);

        // Check for errors
        if ($response === false) {
            return 'Error: ' . curl_error($curl);
        }

        // Close curl
        curl_close($curl);

        // Handle the response
        return  json_decode($response);
    }

    public static function regenerateTallyFile($data)
    {
        $curlUrl = config('app.tally_url').'regeneratefile';
        $username = $data['username'];
        $password = $data['password'];
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $curlUrl,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_HTTPHEADER => array(
                'Content-Type: application/json',
                "username: $username",
                "password: $password",
            ),
        ));

        // Send the request and get the response
        $response = curl_exec($curl);
        // Check for errors
        if ($response === false) {
            echo 'Error: ' . curl_error($curl);
        }

        // Close curl
        curl_close($curl);

        // Handle the response
        return  json_decode($response);
    }
    
    public static function tallyApi($userDetails,$ident,$type,$reqDataAry,$extraData)
    {
        Log::info($type);
        $req=app('App\Http\Controllers\Common\TallyLibraryController')->TallyRequest($ident,$type,$reqDataAry,$extraData);
        $curlUrl = config('app.tally_url').'tallyqueue';
Log::info('Tally Queue Request :'.print_r($req,1));
        
        $curl = curl_init();
        curl_setopt_array($curl, array(
        CURLOPT_URL =>$curlUrl,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'POST',
        CURLOPT_POSTFIELDS =>$req,
        CURLOPT_SSL_VERIFYPEER => false,
        CURLOPT_SSL_VERIFYHOST => false,
        CURLOPT_HTTPHEADER => array(
            "username: $userDetails->username",
            "password: $userDetails->password",
            'Content-Type: application/json'
        ),
        ));
        // Send the request and get the response
        $response = curl_exec($curl);
Log::info('Tally Queue response :'.$response);

        // Check for errors
        if ($response === false) {
            echo 'Error: ' . curl_error($curl);
        }

        // Close curl
        curl_close($curl);

        // Handle the response
        return  json_decode($response);
    }
    

    public static function getTallyConnection($seller_id, $marketplace){
        
        if($marketplace == 'shopify'){
            //Bypass query to shopify database seller table becuase we are not having marketplace_name column in that database
            $sellerDetails =            (array)DB::table(env('DB_DATABASE_SP') . '.spy_credential')->where('id', $seller_id)->where('status', 'Active')->where('module','tlsp')->first();
            if(!empty($sellerDetails)){ //Manually did fix
                $sellerDetails['marketplace_name'] = 'tlsp';
            }
        }else{
            // $sellerDetails = Sellers::where('id', $seller_id)->where('status', 'Active')->where('marketplace_name', '!=', 'tlsp')->whereNull('deleted_at')->first();
            $tallyConnection = TallyConnection::where('seller_id', $seller_id)->where('module', $marketplace)->where('status', 'Active')->first();
        }
        if (empty($tallyConnection)) {
            #Log::info('Invalid seller_id: ' . $seller_id);
            return array('', '');
        }
        
        $module = str_replace('tl', '', $tallyConnection['module']);

        if(!empty($tallyConnection)){
            $tallyConnection['marketplace'] = $module;
        }

        $payoutFields = [];
        $settings = array('sync_settings'=>[],'account_mapping'=>[],'payout_field_mapping'=>[]);

        if ($module == 'az') {
            $database = env('DB_DATABASE_AZ');
            $tallyConnection['module_full_name'] = 'Amazon';
        } 
        
        $settings['sync_settings'] =  Tallysummarysettings::getSummarySyncSettings($seller_id);
        $settings['account_mapping'] = Tallysummaryaccountmapping::getSummaryAccountMapping($seller_id, $marketplace);
        if (!empty($settings['account_mapping'])) {
            $settings['account_mapping']['mapping'] = json_decode($settings['account_mapping']['mapping'], true);
        }
        
        $payoutFields = Tallypayoutfieldmapping::getPayoutField();
        //PENDING TO DISCUS THIS SETTINGS IMPLEMENT IN YALLY OR NOT AM.23-03-2023    
        //Marketplace based settings & account mapping fetch
        // if(in_array($module, array('az'))){
        //  $settings['country_sync_settings_array'] = DB::table($database . '.xo_summary_sync_settings_x')->where('seller_id', $seller_id)->where('status', 'Active')->get();
        //  $settings['country_account_mapping_array'] = DB::table($database . '.summary_account_mapping')->where('seller_id', $seller_id)->where('status', 'Active')->get();
        // }
        
        if(!empty($settings['sync_settings']['account_settings'])){
            $settings['sync_settings']['account_settings'] =  json_decode($settings['sync_settings']['account_settings'],true);
        }
        if(!empty($settings['sync_settings']['multi_tax_code'])){
            $settings['sync_settings']['multi_tax_code'] =  json_decode($settings['sync_settings']['multi_tax_code'],true);
        }
        if(!empty($payoutFields) && $payoutFields->count() > 0){
            $settings['payout_field_mapping'] = json_decode(json_encode($payoutFields->toArray()), true);
        }
        
        //If module is not set that means we didn't find tally details from the table. NS:06-02-2023
        if(!empty($tallyConnection['seller_id']) && !empty($tallyConnection['module'])){
            //Settings are module specific so we are not getting module in it.
            // $settings['sync_settings']['module'] = $tallyConnection['module'];
            // $tallyConnection['customConfig'] = array('featureList' => Xerofeature::fetchFeatureData($tallyConnection));
        } else {
            return array('','');//If tally not connected then reset 
        }
        
        return array($tallyConnection, $settings);
        // return $tallyConnection;
    }

     // All uniq data formate
     public static function DateWithFormat($date = null,$format = null)
     {
         if($date != null)
         {
           $date=date_create($date);
             $date_main = date_format($date,$format);
           return $date_main;
         }
         else
         {
            return "-";
         }
     }

    public function getSettings($seller_id)
    {
        //Fetch module from 
        $sellerDetails = TallyConnection::where('seller_id', $seller_id)->where('status', 'Active')->first();
        
        if(empty($sellerDetails)){
            #Log::info('Invalid seller_id: '. $seller_id);
            return array('','');
        }
        
        $module = str_replace('tl', '', $sellerDetails['module']);
        $tallySyncSetting = '';
        if(!empty($sellerDetails)){
            $sellerDetails['marketplace'] = $module;
        }

        $tallyConnection = $sellerDetails;
        if (!empty($tallyConnection)) {
            $tallyConnection['marketplace'] = $sellerDetails['module'];
            $tallySyncSetting = TallySettings::fetchTallySingleData('seller_id', $tallyConnection['seller_id']);
            if(!empty($tallySyncSetting)){
                $tallySyncSetting['marketplace_name'] = $sellerDetails['module'];
                $tallySyncSetting = $tallySyncSetting->toArray();
                $this->getFormatedSettings($tallySyncSetting);
            }           
        
        } else {
            $tallyConnection['marketplace'] = str_replace('tl', '', $sellerDetails['marketplace_name']);
        }

        if(empty($tallyConnection['seller_id'])){ //Check before fetch features
            #Log::info('Invalid seller_id: ' . $seller_id);
            return array('', '');
        }
        
        if(in_array($sellerDetails['marketplace_name'],array('tlbc', 'tlwc', 'tlss', 'tlmg', 'tlst'))){
            $tallyConnection['customConfig'] = array('featureList' => Xerofeature::fetchFeatureData($tallyConnection));
        }
        
        return array($tallyConnection, $tallySyncSetting);
    }

    public static function getFormatedSettings(&$tallySyncSetting){
        foreach($tallySyncSetting as $key=>$value){
            //Product same having [ braces creates error ] : issue resolved
            if(Helper::isJSON($value)){
                $tallySyncSetting[$key] = json_decode($value,true);
                foreach($tallySyncSetting[$key] as $gateway=>$val){
                    if(!is_array($val) && strpos($val, ',') !== false){
                        $account = explode(",",$val);
                        $tallySyncSetting[$key][$gateway] = array('accountId'=>$account[0],'accountType'=>$account[1]);
                        if($key == 'payment_gateway_map'){
                            $tallySyncSetting['expense_account_types'][] = array('accountType'=>$account[1]);
                        }
                    }
                }
            }
            
            if ($key == 'bank_account') { //Added to sync fees those who have not selected any account in payment_gateway_map. NS:28-06-2022
                $tallySyncSetting['expense_account_types'][] = array('accountType' => $value);
            }
        }
        
        if(!empty($tallySyncSetting['expense_account_types'])){
            $tallySyncSetting['expense_account_types'] = array_unique(array_column($tallySyncSetting['expense_account_types'],'accountType'));
        }else{
            $tallySyncSetting['expense_account_types'] = NULL;
        }
    }
    
}
