<?php

namespace App\Http\Controllers\Summary;
use App\Models\Apierrorlog;
use App\Models\Tallysummary;
use Illuminate\Http\Request;
use App\Models\TallyConnection;
use App\Models\Tallycronongoing;
use App\Models\Tallysummaryaccmappinglog;
use App\Http\Traits\Journal;
use PHPUnit\TextUI\Help;

class JournalvoucherController extends Controller
{
    use Journal;
    protected $bulkDataObjDetail = array();
    protected $commonControllerObj = array();

    public function createJournalVoucher(Request $request)
    {

        $seller_id = $request->seller_id;
        list($tallyConnected, $tallySyncSetting) = $this->getTallyConnection($seller_id, $request->get('module'));
        
        if (!empty($tallyConnected)) {
            $cronOngoingCheck = Tallycronongoing::checkCronOnGoing($tallyConnected['seller_id'], $tallyConnected['module']);
            
            if (!empty($cronOngoingCheck)) {
                return array('code' => '409', 'message' => 'Currently process is going on your store, please try after some time.');
            }
            
            if (empty($request->get('summary_id'))) {
                //For individual order sync we will not create entry.
                Tallycronongoing::createCronOnGoing($tallyConnected['seller_id'], $tallyConnected['module'], ucfirst($tallyConnected['marketplace']) . '_Create_Summary_Invoice_DB_to_TALLY');

                $data = Tallysummary::getTotalJsonDiffRows($tallyConnected);

                if ($data->count() >= 1) {
                    $sellerDetails['seller_id'] = $tallyConnected['seller_id'];
                    $sellerDetails['module'] = $tallyConnected['module'];
                    $sellerDetails['ids'] = implode(', ', array_column($data->toArray(), 'id'));

                    $this->amountMisMatchMail($sellerDetails);
                    
                    $actionlog = array(
                        'error_code' => '10',
                        'error_msg'  => 'Amount mismatch : totalAmount and jsonAmount',
                        'uniqueId'   => $data->first()->toArray()['id'],
                        'data'       => $data->first()->toArray(),
                        'seller_id'  => $tallyConnected['seller_id'],
                        'module'     => $tallyConnected['module']
                    );
                    Apierrorlog::addErrorlog($actionlog);
                    Tallysummary::markOrderPending($tallyConnected, ''); //Mark all summary to pending to avoid other get synced
                    // Tallysummary::markOrderPending($tallyConnected, $sellerDetails['ids']); //Mark only summary to pending having issue in total_amount != json_amount
                    Tallycronongoing::deleteCronOnGoing($tallyConnected['seller_id'], $tallyConnected['module']);
                    return true;
                }

                //We can not create invoice and creditnote in same tally object so need to process separate
                $dbSummaryData = Tallysummary::fetchUnsyncSummary($tallyConnected, $request);
                $positiveSummary =  $dbSummaryData->where('total_amount', '>=', '0');
                $negativeSummary =  $dbSummaryData->where('total_amount', '<', '0');
                
                if ($positiveSummary->count() > 0) {
                    $syncSummary = $this->syncSummaryData($tallyConnected, $tallySyncSetting, $positiveSummary->toArray());
                }
                
                if ($negativeSummary->count() > 0) {
                    $syncSummary = $this->syncSummaryData($tallyConnected, $tallySyncSetting, $negativeSummary->toArray());
                }
            } else {
                //We can not create invoice and creditnote in same tally object so need to process separate
                $dbSummaryData = Tallysummary::fetchUnsyncSummary($tallyConnected, $request);
                if ($dbSummaryData->count() > 0) {
                    $syncSummary = $this->syncSummaryData($tallyConnected, $tallySyncSetting, $dbSummaryData->toArray());
                }
            }
            return $syncSummary; 
        
        }
    }

    public function syncSummaryData($tallyConnected, $tallySyncSetting, $dbSummaries)
    {

        $customFeatureObj = new CustomizefeatureController();

        foreach ($dbSummaries as  $order) {
            if($order['total_amount'] >= 0){
                $type = 'Inv';
            } else {
                $type = 'CN';
            }

            // CALL TO FUNCTION FOR CREATE JOURNAL OR SALES
            $setVoucherType = $this->setVoucherType($tallySyncSetting, $order, $type);

            //CHECK JOURNAL REFERENCE EXITS OR NOT IN TALLY
            if(empty($order['tally_id']) && $order['sync_status'] == "New"){
                $this->checkJournalExitsQueue($tallyConnected, $order, $setVoucherType);
            }else{
                $lineItems = json_decode($order['data'], true);
                $lineItemsArray = $this->formatLineItemData($lineItems);
                $totalLineAmount = $taxLineAmount = 0;
                $lineItem = [];
                foreach ($lineItemsArray as $key => $value) {
                    $accountMappingArray = array('account' => null, 'taxrate' => null);
                    if (!empty((float)$value['amount'])) {
                        if ($tallySyncSetting['sync_settings']['taxcode_disable'] == 'No' && $tallySyncSetting['sync_settings']['tax_method'] == 'line' && strtolower($order['tax_type']) == 'inclusive' && !empty($value['taxAmount'])) {
                            $LineItemAmtValue = $value['amount'] - (float)$value['taxAmount'];
                        } else {
                            $LineItemAmtValue = $value['amount'];
                        }
                        if ($type == 'CN') {
                            $LineItemAmtValue = $LineItemAmtValue * -1;
                        }
                    } else {
                        //Skip that line item having amount 0
                        continue;
                    }
                    $itemData = $customFeatureObj->getItemsWithAccounts($tallySyncSetting, $value['key'], $value['parent'], $value['superParent'], $order);
                    
                    if (!empty($itemData['description']) && strtolower($itemData['description']) != 'other') {
                        if (!empty($value['key'])) { //Append Third level key here
                            $description = is_numeric($value['key']) ? (float)$value['key'] : $value['key'];
                            $lineItem[$key]['description'] = $itemData['description'] . ' - ' . $description;
                        } else {
                            $lineItem[$key]['description'] = $itemData['description'];
                        }
                    } else if (!empty($value['key'])) {
                        $lineItem[$key]['description'] = 'Other - ' . $value['key'];
                    } else if (!empty($value['parent'])) {
                        $lineItem[$key]['description'] = 'Other - ' . $value['parent'];
                    }
                    
                    $lineItem[$key]['Quantity'] = 1;
                    
                    $lineItem[$key]['UnitAmount'] = number_format($LineItemAmtValue, 4, ".", "");
                    $lineItem[$key]['LineAmount'] = number_format($LineItemAmtValue, 4, ".", "");
                    $totalLineAmount += $LineItemAmtValue;
                    
                    if ($tallySyncSetting['sync_settings']['taxcode_disable'] == 'No' && strtoupper($itemData['taxRate']) != 'AUTO') {
                        if (isset($value['taxAmount']) && $tallySyncSetting['sync_settings']['tax_method'] == 'default') {
                            //In ebay we are getting taxAmount = 0 key set so we are passing while create invoice so to proper calculate we have added below condition. NS:16-12-22
                            if(@$tallySyncSetting['sync_settings']['must_apply_tax'] == 'Yes' && $value['taxAmount'] == 0){
                                //Do not set taxAmount in tally object
                            } else {
                                $lineItem[$key]['TaxAmount'] = $value['taxAmount'];
                                if ($type == 'CN') { //Taxamount must be between 0 and lineitem amount. error fixed: NS: 05-12-2022
                                    $lineItem[$key]['TaxAmount'] = $value['taxAmount'] * -1;
                                }
                            }
                            if($order['tax_type'] == 'Exclusive'){ //Getting difference when tax amount is there and to avoid create of adjustment line item of tax amount.
                                $totalLineAmount += $value['taxAmount'];
                            }
                        } else if (isset($value['taxAmount']) && $tallySyncSetting['sync_settings']['tax_method'] == 'line') { //Tax as line item
                            $taxLineAmount += (float)$value['taxAmount'];
                        }
        
                        $taxCode = '';
                        //If taxrate based on feetype is set then we apply else it will take from Tally. becuase we are not passing here
                        if (!empty($itemData['taxRate']) && $tallySyncSetting['sync_settings']['tax_method'] == 'default') {
                            $taxCode = $itemData['taxRate'];
                        } else if ($tallySyncSetting['sync_settings']['tax_method'] == 'line') {
                            //TBD : We will create new column for apply zero_taxcode.
                            $taxCode = 'NONE';
                        }
                        if (!empty($taxCode)) {
                            $accountMappingArray['taxrate'] = $taxCode;
                            $lineItem[$key]['TaxType'] = $taxCode;
                        }
                    }
        
                    if (!empty($itemData['accountCode'])) {
                        $accountMappingArray['account'] = $itemData['accountCode'];
                        $lineItem[$key]['account_code'] = explode(',', $itemData['accountCode'])[0];
                    }
                    
                    //THIS CODE NOT IN USE NOW AM.20-03-2023
                    //Whichever code apply it should be overwrite by set account mapping
                    // $accrefCode = $customFeatureObj->setSummaryAccountMapping($talConnected, $orderData);
                    // $accrefCode = $customFeatureObj->setSummaryAccountMapping($tallyConnected, $order, $value['superParent'],$value['parent'],$value['key']);
                    // if (!empty($accrefCode)) {                    
                    //     $accountMappingArray['account'] = $accrefCode;
                    //     // $lineItem[$key]->setAccountCode($accrefCode);
                    //     $lineItem[$key]['account_code'] = $accrefCode;
                    // }
                        
                    // }
                    // $tallyObject[$indexKey]->addLineItem($$lineItem);
                    // $i++;

                    if (!empty($value['key'])) {
                        $accountMapping[$value['superParent']][$value['parent']][$value['key']] = $accountMappingArray;
                    } else {
                        $accountMapping[$value['superParent']][$value['parent']] = $accountMappingArray;
                    }
                    
                    $lineItem[$key]['is_deemed_positive'] = ($value['amount'] < 0 )? "Yes" : "No";
                    $lineItem[$key]['amount'] = $value['amount'];
                }

                if (!empty((float)$taxLineAmount) && $tallySyncSetting['sync_settings']['tax_method'] == 'line') {
                    $taxLineKey = 'Tax';
                    if ($type == 'CN') {
                        $taxLineAmount = $taxLineAmount * -1;
                    }
                    #$accountMapping[$taxLineKey] = array('account' => null, 'taxrate' => null);
                    $lineItem[$key]['Quantity'] = 1;
                    $lineItem[$key]['UnitAmount'] = number_format($taxLineAmount, 4, ".", "");
                    $lineItem[$key]['LineAmount'] = number_format($taxLineAmount, 4, ".", "");
                    $totalLineAmount += $taxLineAmount;
        
                    //Default account for tax as line item hoy tyare kyu use karvanu puchine confirm karvanu.
                    $itemData = $customFeatureObj->getItemsWithAccounts($tallySyncSetting,'', $taxLineKey, '', $order);
                    if (!empty($itemData['description']) && strtolower($itemData['description']) != 'other') {
                        $lineItem[$key]['description'] = $itemData['description'];
                    } else {
                        $lineItem[$key]['description'] = $taxLineKey;
                    }
        
                    //Tax ni line item par tax na lage etle specifically NONE mokal vanu
                    if ($tallySyncSetting['sync_settings']['taxcode_disable'] == 'No' && !empty($itemData['taxRate']) && strtoupper($itemData['taxRate']) != 'AUTO') {
                        #$accountMapping[$taxLineKey]['taxrate'] = 'NONE';
                        $lineItem[$key]['TaxType'] = 'NONE';
                    }
        
                    if (!empty($itemData['accountCode'])) {
                        #$accountMapping[$taxLineKey]['account'] = $itemData['accountCode'];
                        $lineItem[$key]['account_code'] = explode(',', $itemData['accountCode'])[0];
                    }
                    
                    //THIS CODE NOT IN USE NOW
                    //Whichever code apply it should be overwrite by set account mapping
                    // $accrefCode = $customFeatureObj->setSummaryAccountMapping($tallyConnected, $orderData, $taxLineKey ,'' ,'');
                    // if (!empty($accrefCode)) {                    
                    //     $$lineItem->setAccountCode($accrefCode);
                    // }
                    // $tallyObject[$indexKey]->addLineItem($$lineItem);
                }
                
                //Creditnote bnavti vakhte *-1 karie chie to compare vakhte pan karvu pade. NS:08-09-2022
                if($type == 'CN'){
                    $order['total_amount'] = $order['total_amount'] * -1;
                }
                //Create difference line item of adjustment
                $differenceAmt = number_format($order['total_amount'] - $totalLineAmount, 4, ".", "");
                if (!empty((float)$differenceAmt) && $differenceAmt != 0) {
                    $differenceKey = 'Adjustment';
                    #$accountMapping[$differenceKey] = array('account' => null, 'taxrate' => null);
                    $key++;
                    $lineItem[$key]['Quantity'] = 1;
                    $lineItem[$key]['UnitAmount'] = $differenceAmt;
                    $lineItem[$key]['LineAmount'] = $differenceAmt;
                    $itemData = $customFeatureObj->getItemsWithAccounts($tallySyncSetting, '', $differenceKey, '',  $order);
                    $totalLineAmount += $differenceAmt;
                    //Adjustment ni line item par tax na lage etle specifically NONE mokal vanu
                    if ($tallySyncSetting['sync_settings']['taxcode_disable'] == 'No' && !empty($itemData['taxRate']) && strtoupper($itemData['taxRate']) != 'AUTO') {
                        #$accountMapping[$differenceKey]['taxrate'] = 'NONE';
                        $lineItem[$key]['TaxType'] = 'NONE';
                    }
                    
                    if (!empty($itemData['description']) && strtolower($itemData['description']) != 'other') {
                        $lineItem[$key]['description'] = $itemData['description'];
                    } else {
                        $lineItem[$key]['description'] = $differenceKey;
                    }
        
                    if (!empty($itemData['accountCode'])) {
                        #$accountMapping[$differenceKey]['account'] = $itemData['accountCode'];
                        $lineItem[$key]['account_code'] = explode(',', $itemData['accountCode'])[0];
                    }

                    $lineItem[$key]['is_deemed_positive'] = ($differenceAmt < 0 )? "Yes" : "No";
                    $lineItem[$key]['amount'] = $differenceAmt;

                    //THIS CODE NOT IN USE
                // Whichever code apply it should be overwrite by set account mapping
                    // $accrefCode = $customFeatureObj->setSummaryAccountMapping($tallyConnected, $orderData, $differenceKey, '', '');
                    // if (!empty($accrefCode)) {                    
                    //     $$lineItem->setAccountCode($accrefCode);
                    // }
        
                    // $tallyObject[$indexKey]->addLineItem($$lineItem);
                }

                
                //Settlement lineitem
                $totalLineAmount = number_format(($totalLineAmount), 4, ".", "");
                if (!empty((float)$totalLineAmount) && $totalLineAmount != 0) {
                    if($type == "CN"){
                        $settlementAmount = $order['total_amount'];
                    }else{
                        $settlementAmount = $order['total_amount'] * -1;
                    }
                    
                    $differenceKey = 'Settlement';
                    #$accountMapping[$differenceKey] = array('account' => null, 'taxrate' => null);
                    $key++;
                    $lineItem[$key]['Quantity'] = 1;
                    $lineItem[$key]['UnitAmount'] = $totalLineAmount;
                    $lineItem[$key]['LineAmount'] = $totalLineAmount;
                    $itemData = $customFeatureObj->getItemsWithAccounts($tallySyncSetting, '', $differenceKey, '',  $order);

                    if ($settlementAmount != 0) {
                        //Adjustment ni line item par tax na lage etle specifically NONE mokal vanu
                        if ($tallySyncSetting['sync_settings']['taxcode_disable'] == 'No' && !empty($itemData['taxRate']) && strtoupper($itemData['taxRate']) != 'AUTO') {
                            #$accountMapping[$differenceKey]['taxrate'] = 'NONE';
                            $lineItem[$key]['TaxType'] = 'NONE';
                        }
                        if (!empty($itemData['description']) && strtolower($itemData['description']) != 'other') {
                            $lineItem[$key]['description'] = $itemData['description'];
                        } else {
                            $lineItem[$key]['description'] = $differenceKey;
                        }
                        
                        //Overwrite account and fetch from settings if not found then do not set and let summary goes to pending
                        $itemData['accountCode'] = $customFeatureObj->getSettlementAccounts($tallyConnected,$tallySyncSetting, $order);
                        if (!empty($itemData['accountCode'])) {
                            #$accountMapping[$differenceKey]['account'] = $itemData['accountCode'];
                            $lineItem[$key]['account_code'] = explode(',', $itemData['accountCode'])[0];
                        }
                        $lineItem[$key]['is_deemed_positive'] = (number_format($settlementAmount, 4, ".", "") < 0 )? "Yes" : "No";
                        $lineItem[$key]['amount'] = number_format($settlementAmount, 2, ".", "");
                    }
                    //THIS CODE NOT IN USE
                    // Whichever code apply it should be overwrite by set account mapping
                    // $accrefCode = $customFeatureObj->setSummaryAccountMapping($tallyConnected, $orderData, $differenceKey, '', '');
                    // if (!empty($accrefCode)) {                    
                    //     $$lineItem->setAccountCode($accrefCode);
                    // }
        
                    // $tallyObject[$indexKey]->addLineItem($$lineItem);
                }

                $invoiceDocNumberArray = explode('_', $order['document_number']);
                if ($tallySyncSetting['sync_settings']['prefix_required'] == 'Y') {
                    if (empty(is_numeric($invoiceDocNumberArray[1][0]))) {
                        $invReference = $invoiceDocNumberArray[0] . '_' . $invoiceDocNumberArray[1];
                    } else { //To handle AMZ_20220802
                        $invReference = $invoiceDocNumberArray[0];
                    }
                } else {
                    $invReference = $invoiceDocNumberArray[0];
                }

                if(!empty($order['reference'])){
                    $invReference = $order['reference'];
                } else if(empty($order['start_date']) || empty($order['end_date'])){
                    $invReference = $order['document_number']; //If start_date or end date is missing then set docnumber as reference. NS:31-08-2022
                } else {
                    //If start_date and end_date are same then pass only date in reference instead of from date to date
                    if(date("d M Y", strtotime($order['start_date'])) == date("d M Y", strtotime($order['end_date']))){
                        $invReference .= ' ' . date("d M Y", strtotime($order['start_date']));
                    } else {
                        $invReference .= ' From ' . date("d M Y", strtotime($order['start_date'])) . ' to ' . date("d M Y", strtotime($order['end_date']));
                    }
                }
                $dateInstance = date("Ymd", strtotime($order['invoice_date']));
                $journalArray = [
                    "REFERENCE" => $order['document_number'],
                    "NARRATION" => $invReference,
                    "DATE" => $dateInstance,
                    "EFFECTIVEDATE" => $dateInstance,//CHANGE THIS VALUE NOW SET STATIC
                    "lineItem" => 
                        $lineItem
                    
                ];
                $generatQueue = $this->createTallyQueue($tallyConnected, $journalArray, $order, $setVoucherType, $accountMapping);
                // Tallysummaryaccmappinglog::insertAccountMappingData($tallyConnected, $tallySyncSetting, $order['id'], $accountMapping);
            }
            // }
            continue;
        }
        
        return array('code' => '200', 'message' => "Queue created successfully!");
    }

    public function createTallyQueue($tallyConnected, $journalArray, $summaryData, $setVoucherType, $accountMapping)
    {
        $connectionDetails = TallyConnection::checkTallyConnection($tallyConnected['seller_id'], $tallyConnected['module']);
        if(!empty($connectionDetails)){
            // $ident="LedgerCreate-".$store_id."-".$connectionDetails->id;
           
            $journalItemArray = [];
            foreach($journalArray['lineItem'] as $key => $journalValue){ //SET ALLLEDGERENTRIES.LIST ARRAY
                $journalItemArray[$key]['LEDGERNAME'] = $journalValue['account_code'];
                $journalItemArray[$key]['ISDEEMEDPOSITIVE'] = $journalValue['is_deemed_positive'];
                $journalItemArray[$key]['AMOUNT'] = $journalValue['amount'];
            }
            if(!empty($summaryData['tally_id'])){
                $attributeArray = [
                        "DATE" => $journalArray['DATE'],
                        "TAGNAME" => "MASTER ID",
                        "TAGVALUE" => $summaryData['tally_id'],
                        "VCHTYPE" => $setVoucherType,//SET VOUCHER TYPE (SLAES, CREADIT NOTE, JOURNAL) GET BY FUNCTION
                        "ACTION" => "Alter",
                ];
                $vaoucherTypeAction = "Update";
            }else{
                $attributeArray = [
                    "VCHTYPE" => $setVoucherType,//SET VOUCHER TYPE (SLAES, CREADIT NOTE, JOURNAL) GET BY FUNCTION
                    "ACTION" => "Create",
                ];
                $vaoucherTypeAction = "Create";
            }
            $replaceVoucherType = str_replace(' ','', $setVoucherType).$vaoucherTypeAction;
            if($setVoucherType == "Sales"){
                $replaceVoucherType = str_replace(' ','', $setVoucherType)."Voucher".$vaoucherTypeAction;
            }
            $ident = $replaceVoucherType."-".$tallyConnected['seller_id'].'-'.$summaryData['id'];
            
            list($reqDataAry,$extraData) = $this->createJournalSummaryXml($connectionDetails,$attributeArray,$setVoucherType,$journalArray,$journalItemArray,$summaryData,'summary');

            $response = $this->tallyApi($connectionDetails,$ident,$replaceVoucherType,$reqDataAry,$extraData);
            if(!empty($response->success)){
                Tallysummary::updateSummaryStatus($extraData, "OJV");
            }
            $data = $this->handleResponse($response);
        }else{
            $data['success'] = false;
            $data['message'] = "Tally not connected for this store";
            $code= 401;
        }
        return $data;
    }

    //CHECK BY REFERENCE WISE LEDGER IS EXITS IN TALLY OR NOT QUEUE
    public function checkJournalExitsQueue($tallyConnected, $summaryData, $setVoucherType)
    {
        $connectionDetails = TallyConnection::checkTallyConnection($tallyConnected['seller_id'], $tallyConnected['module']);
        if(!empty($connectionDetails)){
            $ident = "VoucherCollection-".$tallyConnected['seller_id'].'-'.$summaryData['id'];
            
            list($reqDataArry,$extraData) = $this->getJournalSummaryXml($connectionDetails,$summaryData,$setVoucherType);

            $response = $this->tallyApi($connectionDetails,$ident,"VoucherCollection",$reqDataArry,$extraData);
            if(!empty($response->success)){
                Tallysummary::updateSummaryStatus($extraData, "ORCHECK");
            }
            $data = $this->handleResponse($response);
        } else{
            $data['success'] = false;
            $data['message'] = "Tally not connected for this store";
            $code= 401;
        }
        return $data;
    }
    
    public function amountMisMatchMail($sellerDetails){
        $content = view('mail', ['seller'=> $sellerDetails]); // $this->load->view('email', '', true);

        $maildata['email_to'] = 'sanjay@parextech.com';
        $maildata['email_to_name'] = 'Sanjay Patel';
        $maildata['email_from'] = 'apps@parextech.com';
        $maildata['email_from_name'] = 'Parex Bridge Support';
        $maildata['subject'] = 'Summary sync stop due to amount mismatch';
        $maildata['body'] = (string)$content;
        $maildata['text'] = 'Thank you for using QB Bridge';
        $maildata['tag'] = 'QuickBooks Bridge Support';
        $maildata['reply_to'] = 'apps@parextech.com';
        $maildata['cc'] = '';
        
        // $mailResponse = Helper::sendmail($maildata['email_to'],$maildata['email_to_name'],$maildata['email_from'],$maildata['email_from_name'],$maildata['subject'],$maildata['body'],$maildata['text'],$maildata['tag'],$maildata['reply_to'],$maildata['cc']);
        // Maillog::saveMailLog($maildata,$mailResponse);
        
    }
}
