<?php
namespace App\Http\Traits;
use App\Helpers\Helper;
use App\Models\TallyConnection;
use App\Models\Accounts as AccountsModel;
use App\Models\TallyDdraftSettings;
use App\Http\Controllers\Common\AccountsController;
use Log;
use Request;
/*
Manali Patel - 15-09-2023
This trait is used to form xml requests to create and fetch accounts.
*/
trait Accounts
{
    public function getAccountsExtraData($connectionDetails){
        $extraData = [
            "seller_id" => $connectionDetails->seller_id,
            "module" => $connectionDetails->module,
            "company" => $connectionDetails->company_name,
            "name" => $connectionDetails->username
        ];
        return $extraData;
    }

    public function getAccountsXml($connectionDetails)
    {
        $reqDataAry = [
            "HEADER" => [
                "VERSION" => 1,
                "TALLYREQUEST" => "Export",
                "TYPE" => "Collection",
                "ID" => "Ledger Collection",
            ],
            "BODY" => [
                "DESC" => [
                    "STATICVARIABLES" => [
                        "SVEXPORTFORMAT" => '$$SysName:XML',
                        "SVCURRENTCOMPANY" => $connectionDetails->company_name,
                    ],
                    "TDL" => [
                        "TDLMESSAGE" => [
                            "COLLECTION" => [
                                "@attributes" => [
                                    "NAME" => "Ledger Collection",
                                    "ISMODIFY" => "No",
                                    "ISFIXED" => "No",
                                    "ISINITIALIZE" => "No",
                                    "ISOPTION" => "No",
                                    "ISINTERNAL" => "No",
                                ],
                                "TYPE" => "Ledger",
                                "FETCH" => "NAME,PINCODE,COUNTRYNAME,INCOMETAXNUMBER,TAXTYPE,COUNTRYOFRESIDENCE,LEDSTATENAME,PARENT,GUID",
                            ],
                        ],
                    ],
                ],
            ],
        ];
        $extraData = $this->getAccountsExtraData($connectionDetails);
        return array($reqDataAry,$extraData);
    }

    public function createAccountExtraData($connectionDetails,$account){
        $extraData = [
            "seller_id" => $connectionDetails->seller_id,
            "module" => $connectionDetails->module,
            "company" => $connectionDetails->company_name,
            "name" => $account['name'],
            "parent" => $account['parent'],
            "address" => @$account['address'],
            "pincode" => @$account['pincode'],
            "country" => @$account['country'],
            "taxtype" => @$account['taxtype'],
            "state" => @$account['state']
        ];
        return $extraData;
    }

    public function createAccountXml($account,$connectionDetails){
        $reqDataAry = [
            "@attributes" => [
                "ACTION" => "CREATE"
            ],
            "NAME" => $account['name'],
            "ADDRESS.LIST" => [
                "@attributes" => [
                    "TYPE" => "String"
                ],

                "ADDRESS" => isset($account['address']) ? $account['address'] : '',
            ],

            "PINCODE" => isset($account['pincode']) ? $account['pincode'] : '',
            "COUNTRYNAME" => isset($account['country']) ? $account['country'] : '',
            "PARENT" => $account['parent'],
            "TAXTYPE" => isset($account['taxtype']) ? $account['taxtype'] : '',
            "COUNTRYOFRESIDENCE" => isset($account['countryofresidence']) ? $account['countryofresidence'] : '',
            "LEDSTATENAME" => isset($account['state']) ? $account['state'] : '',
            "BILLCREDITPERIOD" => isset($account['billcreditperiod']) ? $account['billcreditperiod'] : '',
        ];
        $extraData = $this->createAccountExtraData($connectionDetails,$account);
        return array($reqDataAry,$extraData);
    }

    public static function LedgerCreateResponse($res)
    {
        $response = json_decode($res['extra_data']);
        $data = [
            'name'=>  isset($response->name) ? $response->name : '',
            'parent'=> isset($response->parent) ? $response->parent : '',
            'country'=> isset($response->country) ? $response->country : '',
            'state' => isset($response->state) ? $response->state : '',
            'tax_type'=> isset($response->taxtype) ? $response->taxtype : '',
            'seller_id' => $response->seller_id,
            'module' => $response->module
        ];
        TallyDdraftSettings::updateFlag($response->seller_id,$response->module);
        //AccountsModel::UpdateAccountData($response->name, $response->seller_id, $response->module,$data);
        // Accounts::UpdateAccountData($response->name,$response->country,$response->seller_id,$response->state,$response->taxtype, $response->module,$response->parent);
        
        $requestAcc = new \Illuminate\Http\Request();
        $requestAcc->replace(['seller_id' =>  $response->seller_id ,'module'=>$response->module]);
        $account = new AccountsController();
        $account->getAccounts($requestAcc);

        return '{"success": 1, "message":"Success"}';
    }

    /*
        delete all existing accounts and then create new accounts 
    */
    public static function ledgerListResponse($res)
    {   
        $response = json_decode($res['response_data']);
        $extraData = json_decode($res['extra_data']);
        Log::info('Accounts List : '.print_r($response,1));
        Log::info('Accounts List : '.print_r($extraData,1));
        if(!empty($response->BODY->DATA->COLLECTION->LEDGER)){
            
            AccountsModel::deleteAllAccounts($extraData->seller_id, $extraData->module);
            foreach ($response->BODY->DATA->COLLECTION->LEDGER as $key => $LEDGER) {
                $ledgerArray = (array) $LEDGER;
                $name = (!empty($ledgerArray['@attributes']->NAME)) ? $ledgerArray['@attributes']->NAME : "";
                $country = (!empty($ledgerArray['COUNTRYNAME'])) ? $ledgerArray['COUNTRYNAME'] : "";
                $state = (!empty($ledgerArray['LEDSTATENAME'])) ? $ledgerArray['LEDSTATENAME'] : "";
                $tax_type = (!empty($ledgerArray['TAXTYPE'])) ? $ledgerArray['TAXTYPE'] : "";
                $parent = (!empty($ledgerArray['PARENT'])) ? $ledgerArray['PARENT'] : "";
                $GUID = (!empty($ledgerArray['GUID'])) ? $ledgerArray['GUID'] : "";
                //CREATE ACCOUNTS DATA
                AccountsModel::createAccountData($name, $country, $extraData->seller_id, $state, $tax_type, $parent, $GUID, $extraData->module);
            }
            TallyConnection::updateAccountFlag($extraData->seller_id,$extraData->module);
        }
        return '{"success": 1, "message":"Success"}';

    }
}
