<?php
namespace App\Http\Traits;
use App\Helpers\Helper;
use App\Models\Tallycompany;
use App\Models\TallyConnection;
use App\Http\Controllers\Common\AccountsController;
use App\Http\Controllers\Common\ItemController;
use Illuminate\Http\Request as HttpRequest;
use Log;

/*
Manali Patel - 15-09-2023
This trait is used to form xml requests to create and fetch tax.
*/
trait Company
{
    public function updateCompanyExtraData($connectionDetails,$reqDataAry){
        $extraData = [
                "seller_id" => $connectionDetails->seller_id,
                "module" => $connectionDetails->module,
                "company_name" => $reqDataAry['companyName'],
                "name" => $connectionDetails->username,
                "id" => $connectionDetails->id,
                "tally_connection_id" => $connectionDetails->tallyConnectionId,
                "company_GUID" => $reqDataAry['companyGUID'],
            ];

            return $extraData;
    }

    public function updateCompanyXml($connectionDetails,$company_name,$company_id){
        $reqDataAry = [
                "companyGUID" => $company_id,
                "companyName" => $company_name,
            ];
        $extraData = $this->updateCompanyExtraData($connectionDetails,$reqDataAry);
        return array($reqDataAry,$extraData);
    }

    public function getCompanyListExtraData($connectionDetails){
        $extraData = [
            "seller_id" => $connectionDetails->seller_id,
            "module" => $connectionDetails->module,
        ];
        return $extraData;
    }

    public function getCompanyListXml($connectionDetails){

        $reqDataAry = [
            "HEADER" => [
                "VERSION" => 1,
                "TALLYREQUEST" => "Export",
                "TYPE" => "Collection",
                "ID" => "Company Collection",
            ],
            "BODY" => [
                "DESC" => [
                    "STATICVARIABLES" => [
                        "SVEXPORTFORMAT" => '$$SysName:XML',
                    ],
                    "TDL" => [
                        "TDLMESSAGE" => [
                            "COLLECTION" => [
                                "@attributes" => [
                                    "NAME" => "Company Collection",
                                    "ISMODIFY" => "No",
                                    "ISFIXED" => "No",
                                    "ISINITIALIZE" => "No",
                                    "ISOPTION" => "No",
                                    "ISINTERNAL" => "No",
                                ],
                                "TYPE" => "Company",
                                "FETCH" => "NAME,*",
                            ],
                        ],
                    ],
                ],
            ],
        ];
        $extraData = $this->getCompanyListExtraData($connectionDetails);
        return array($reqDataAry,$extraData);
    }

    public static function CompanyListResponse($res)
    {   
        $response=json_decode($res['response_data']);
        $extraData = json_decode($res['extra_data']);
        Log::info('company list response: ' . print_r($response,1));
        Log::info('company list extra data: ' . print_r($extraData,1));
        if(!empty($response->BODY->DATA->COLLECTION->COMPANY)){
            
            $companyData = [];
            $i = 0;
            if(is_object($response->BODY->DATA->COLLECTION->COMPANY)){
                $CompanyList = [$response->BODY->DATA->COLLECTION->COMPANY];
            }
            else{
                $CompanyList = $response->BODY->DATA->COLLECTION->COMPANY;
            }
            Tallycompany::deleteAllCompany($extraData->seller_id, $extraData->module);
            foreach ($CompanyList as $key => $COMPANY) {
                $companyArray = (array) $COMPANY;
                $companyData[$i]['seller_id'] = (!empty($extraData->seller_id)) ?$extraData->seller_id : "";
                $companyData[$i]['module'] = (!empty($extraData->module)) ? $extraData->module : "";
                $companyData[$i]['guid'] = (!empty($companyArray['GUID'])) ? $companyArray['GUID'] : "";
                $companyData[$i]['name'] = (!empty($companyArray['@attributes']->NAME)) ? $companyArray['@attributes']->NAME : "";
                $companyData[$i]['country'] = (!empty($companyArray['COUNTRYNAME'])) ? $companyArray['COUNTRYNAME'] : "";
                $companyData[$i]['vat_dealer_type'] = (!empty($companyArray['VATDEALERTYPE'])) ? $companyArray['VATDEALERTYPE'] : "";
                $companyData[$i]['company_data'] = json_encode($COMPANY) ;
                $i++;
            }
            Tallycompany::createCompanyData($companyData);
            $updateData = ['company_list_flag' => 1];
            Tallycompany::updateCompanyData($extraData->seller_id, $extraData->module, $updateData);
        }
        return '{"success": 1, "message":"Success"}';

    }

    public static function CompanyUpdateResponse($res)
    {   
        Log::info('company update response : '.print_r($res,1));
        $response = json_decode($res['response_data'],true);
        $extraData = json_decode($res['extra_data'],true);
        if ($response['success'] == 1) {
            $updateData = ['company_update_flag' => 1];
            Tallycompany::updateCompanyData($extraData['seller_id'], $extraData['module'], $updateData);
            Log::info('company details update response : '.print_r($response,1));
            Log::info('company details update extradata: '.print_r($extraData,1));
            $updateConnectionData = ['company_guid' => $extraData['company_GUID'], 'company_name' => $extraData['company_name']];
            TallyConnection::updateConnection($extraData['seller_id'], $extraData['module'], $extraData['tally_connection_id'], $updateConnectionData);
            $requestAcc = new \Illuminate\Http\Request();
            $requestAcc->replace(['seller_id' =>  $extraData['seller_id'] ,'module'=>$extraData['module']]);
            $account = new AccountsController();
            $account->getAccounts($requestAcc);
            $items = new ItemController();
            $items->getItems($requestAcc);
        }
        return '{"success": 1, "message":"Success"}';

    }
}
