<?php
namespace App\Http\Traits;
use App\Helpers\Helper;
use App\Models\TallyConnection;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Individual\InvoiceController;
use App\Models\TallyOrders;
use App\Models\Tallyproduct;
use Illuminate\Http\Request;
use Log;

/*
Manali Patel - 15-09-2023
This trait is used to form xml requests to create and fetch accounts.
*/
trait Item
{
    
    public function getItemExtraData($connectionDetails,$orderData,$lineItemData){
        $extraData = [
            "seller_id" => $connectionDetails['seller_id'],
            "module" => $connectionDetails['module'],
            "company" => $connectionDetails->company_name,
            "name" => $connectionDetails->username,
            "order_id" => isset($orderData['id']) ? $orderData['id'] : '',
            "order" => $orderData,
            "line_items" => $lineItemData,
        ];
        return $extraData;
    }

    public function getItemXml($connectionDetails,$orderData,$lineItemData,$firstName,$newName,$type = 'all'){
        $reqDataArry = [
            "HEADER" => [
                "VERSION" => "1",
                "TALLYREQUEST" => "Export",
                "TYPE" => "Collection",
                "ID" => "Stock Item Collection",
            ],
            "BODY" => [
                "DESC" => [
                    "STATICVARIABLES" => [
                        "SVEXPORTFORMAT" => '$$SysName:XML',
                        "SVCURRENTCOMPANY" => $connectionDetails->company_name
                    ],
                
                    "TDL" => [
                        "TDLMESSAGE" => [
                            "COLLECTION" => [
                                "@attributes" => [
                                    "NAME" => "Stock Item Collection",
                                    "ISMODIFY" => "No",
                                    "ISFIXED" => "No",
                                    "ISINITIALIZE" => "No",
                                    "ISOPTION" => "No",
                                    "ISINTERNAL" => "No",
                                ],
                                "TYPE" => "Stock Item",
                                "FETCH" => "NAME"
                            ],
                        ],
                    ],
                ],
            ],
        ];
        if($type != 'all'){
            $reqDataArry['BODY']['DESC']['TDL'] = [
                "TDLMESSAGE" => [
                    "COLLECTION" => [
                        "FILTER" => "TYPEOFFILTER"
                    ]
                ]
            ];
            $reqDataArry['BODY']['DESC']['TDL'] = [
                "SYSTEM" => [
                    "@attributes" => [
                        "TYPE" => "Formulae",
                        "NAME" => "TYPEOFFILTER",
                    ],
                    "@value" => '$NAME Equal '.$firstName.'"'.$newName,
                ]
            ];
        }
        $extraData = $this->getItemExtraData($connectionDetails,$orderData,$lineItemData);
        return array($reqDataArry,$extraData);
    }

    //WHEN JOURNAL CREATED UPDATE SUMMARY TALLY ID IN SUMMARY TABLE
    public static function ItemsCollection($res)
    {   
        $response = json_decode($res['response_data']);
        Log::info("Item collection DAta1".print_r( $response,1));
        $extraData = json_decode($res['extra_data']);
        if(!empty($response->BODY->DATA->COLLECTION->STOCKITEM)){
            $itemList = $response->BODY->DATA->COLLECTION->STOCKITEM;
            if(is_object($response->BODY->DATA->COLLECTION->STOCKITEM)){
                $itemList = [$response->BODY->DATA->COLLECTION->STOCKITEM];
            }
            else{
                $itemList = $response->BODY->DATA->COLLECTION->STOCKITEM;
            }
            $item = [];
            foreach ($itemList as $key => $Items) {
                $ledgerArray = (array) $Items;
                if(in_array($ledgerArray['@attributes']->NAME,$extraData->line_items)){
                    $key = array_search($ledgerArray['@attributes']->NAME, $extraData->line_items);
                    if (false !== $key) {
                        /*needs to save this in db*/
                        $dbProduct = Tallyproduct::checkExistProduct($extraData->seller_id,$extraData->module,$extraData->line_items[$key]);
                        if(empty($dbProduct)){
                            $i=0;
                            $items[$i]['sku'] = $extraData->line_items[$key];
                            $items[$i]['seller_id'] = $extraData->seller_id;
                            $items[$i]['module'] = $extraData->module;

                            Tallyproduct::createProductData($items);
                        }
                        unset($extraData->line_items[$key]);
                    }
                }else{
                    if(empty($extraData->line_items)){
                        $dbProduct = Tallyproduct::checkExistProduct($extraData->seller_id,$extraData->module,$ledgerArray['@attributes']->NAME);
                        if(empty($dbProduct)){
                            $i=0;
                            $items[$i]['sku'] = $ledgerArray['@attributes']->NAME;
                            $items[$i]['seller_id'] = $extraData->seller_id;
                            $items[$i]['module'] = $extraData->module;

                            Tallyproduct::createProductData($items);
                        }
                    }
                }
            }
        }
        $connectionDetails = TallyConnection::checkTallyConnection($extraData->seller_id, $extraData->module);
        if(!empty($connectionDetails)){
            $ident = "ItemCreate-".$extraData->seller_id;
            $item = [];
            if(!empty($extraData->line_items)){
                foreach($extraData->line_items as $key => $lineItem){
                    $item[] = [ 
                            "STOCKITEM" => [
                                "@attributes" => [
                                    "ACTION" => "CREATE"
                                ],
                            
                                "NAME" => $lineItem,
                                "BASEUNITS" => "Nos",
                                "GSTAPPLICABLE" =>  "&#4; Applicable",
                                "NAME.LIST" => [
                                    "@attributes" => [
                                        "TYPE" => "String"
                                    ],

                                    "NAME" => $lineItem,
                                ]
                            ],
                        ];
                }
                $item = array_values($item);
                if(!empty($item)){
                    $reqDataArry = [
                        "HEADER" => [
                            "VERSION" => "1",
                            "TALLYREQUEST" => "IMPORT",
                            "TYPE" => "DATA",
                            "ID" => "ALL MASTERS",
                        ],
                        "BODY" => [
                            "DESC" => [
                                "STATICVARIABLES" => [
                                    "SVCURRENTCOMPANY" => $connectionDetails->company_name
                                ],
                            
                                "DATA" => [
                                    "TALLYMESSAGE" => 
                                        $item
                                    ,
                                ],
                            ],
                        ],
                    ];
                    $extraData = [
                        "seller_id" => $extraData->seller_id,
                        "module" => $extraData->module,
                        "company" => $connectionDetails->company_name,
                        "name" => $connectionDetails->username,
                        "order_id" => $extraData->order_id,
                        "line_items" => $extraData->line_items,
                        "order" => $extraData->order,
                    ];
                    $response = Controller::tallyApi($connectionDetails,$ident,"ItemCreate",$reqDataArry,$extraData);
                    if(!empty($response->success)){
                        $data['success'] = true;
                        $data['message'] = (isset($response->message))?$response->message:$response;
                    }else{
                        $actionlog = array(
                            'error_code' => '400',
                            'error_msg'  => $response->message,
                            'data'       => $response,
                            'seller_id'  => $extraData['seller_id'],
                            'module'     => $extraData['module']
                        );
                        Apierrorlog::addErrorlog($actionlog);
                        $data['success'] = false;
                        $data['message'] = (isset($response->message))?$response->message:$response;
                    }
                }
            }else {
                if(!empty($extraData->order)){
                    $controllerObj = new Controller();
                    list($tallyConnected, $tallySyncSetting) = $controllerObj->getSettings($extraData->seller_id);
                    $data = ['order_number' => $extraData->order->order_number];
                    $request = Request::create('/example', 'POST', $data);
                    $dbOrders = TallyOrders::fetchUnsyncOrders($tallyConnected, $request, 'create');
                    $orderObj = new InvoiceController();
                    $orderObj->syncOrderData($tallyConnected, $tallySyncSetting, $dbOrders);
                }
            }

        } else{
            $data['success'] = false;
            $data['message'] = "Tally not connected for this store";
            $code= 401;
        }
        return '{"success": 1, "message":"Success"}';

    }

    public static function ItemsCreate($res)
    {
        Log::info("Item create DAta1".print_r( $res,1));
        $response = json_decode($res['response_data']);
        $extraData = json_decode($res['extra_data']);
        if(isset($response->BODY->DATA->IMPORTRESULT) && ($response->BODY->DATA->IMPORTRESULT->CREATED == 2 || $response->BODY->DATA->IMPORTRESULT->ALTERED == 2) || ($response->BODY->DATA->IMPORTRESULT->CREATED == 1 || $response->BODY->DATA->IMPORTRESULT->ALTERED == 1)){
            $i=0;
            foreach($extraData->line_items as $value){
                $items[$i]['sku'] = $value;
                $items[$i]['seller_id'] = $extraData->seller_id;
                $items[$i]['module'] = $extraData->module;
                $i++;
            }
            Tallyproduct::createProductData($items);
        }
        TallyOrders::where('id',$extraData->order_id)->where('module', $extraData->module)->where('seller_id', $extraData->seller_id)->update(['sync_flag' => "ORCH"]);
        if($extraData->order_id != ''){
            /*Put order check queue*/
            $orderObj = new InvoiceController();
            $orderObj->checkOrderExitsQueue($extraData->order_id);
        }
    }

    public function getProductNametoMatch($tallySyncSetting,$lineItem){
        $product_name_to_match = '';

        if($tallySyncSetting['product_matching'] == 's2n'){
            $product_name_to_match = $lineItem['sku'];
        }elseif($tallySyncSetting['product_matching'] == 'name'){
            $product_name_to_match = substr($lineItem['name'], 0, config('mastertallyconstant.productnametruncatelimit'));
        }
        return $product_name_to_match;
    }
}
