<?php
namespace App\Http\Traits;
use App\Helpers\Helper;
use App\Models\TallyOrders;
use App\Models\Tallyrefunds;
use App\Models\Tallypayments;
use App\Models\Apierrorlog;
use App\Http\Controllers\Individual\InvoiceController;
use App\Http\Controllers\Individual\ReceiptController;
use Log;

/*
Manali Patel - 15-09-2023
This trait is used to form xml requests to create and fetch accounts.
*/
trait Journal
{
    public function setVoucherType($tallySyncSetting, $order, $type)
    {
        if($type == 'JOURNAL'){
            $types = "Journal";
        }elseif($type == 'CN'){
            $types = "Credit Note";
        }else{
            if($tallySyncSetting['sync_type'] == "SALES"){
                if($order['total_amount'] >= 0){
                    $types = "Sales";
                }else if($order['total_amount'] < 0){
                    $types = "Credit Note";
                }
            }else{
                $types = "Journal";
            }
        }
        return $types;

    }

    public function getJournalSummaryExtraData($connectionDetails,$summaryData){
        $extraData = [
            "seller_id" => $connectionDetails['seller_id'],
            "module" => $connectionDetails['module'],
            "company" => $connectionDetails->company_name,
            "name" => $connectionDetails->username,
            "summary_id" => $summaryData['id'],
            "order" => $summaryData,
        ];
        return $extraData;
    }

    public function getJournalSummaryXml($connectionDetails,$summaryData,$setVoucherType){
        $reqDataArry = [
            "HEADER" => [
                "VERSION" => "1",
                "TALLYREQUEST" => "Export",
                "TYPE" => "Collection",
                "ID" => "Voucher Collection",
            ],
            "BODY" => [
                "DESC" => [
                    "STATICVARIABLES" => [
                        "SVEXPORTFORMAT" => '$$SysName:XML',
                        "SVCURRENTCOMPANY" => $connectionDetails->company_name
                    ],
                
                    "TDL" => [
                        "TDLMESSAGE" => [
                            "COLLECTION" => [
                                "@attributes" => [
                                    "NAME" => "Voucher Collection",
                                    "ISMODIFY" => "No",
                                    "ISFIXED" => "No",
                                    "ISINITIALIZE" => "No",
                                    "ISOPTION" => "No",
                                    "ISINTERNAL" => "No",
                                ],
                                "TYPE" => "Voucher",
                                "FETCH" => "VOUCHERTYPENAME,REFERENCE,DATE,NARRATION,PERSISTEDVIEW,EFFECTIVEDATE,ALLLEDGERENTRIES.LIST.*",
                                "FILTER" => "TYPEOFFILTER",
                            ],
                           "SYSTEM" => [
                                "@attributes" => [
                                    "TYPE" => "Formulae",
                                    "NAME" => "TYPEOFFILTER",
                                ],
                                "@value" => '$VOUCHERTYPENAME Equal "'.$setVoucherType.'" AND $REFERENCE Equal "'.$summaryData['document_number'].'"',
                            ],
                        ],
                    ],
                ],
            ],
        ];
        $extraData = $this->getJournalSummaryExtraData($connectionDetails,$summaryData);
        return array($reqDataArry,$extraData);
    }

    public function createJournalSummaryExtraData($connectionDetails,$summaryData,$journalArray,$accountMapping = array(),$sync_for = 'summary'){
        $extraData = [
            "seller_id" => $connectionDetails['seller_id'],
            "module" => $connectionDetails['module'],
            "company" => $connectionDetails->company_name,
            "name" => $connectionDetails->username,
            "summary_id" => $summaryData['id'],
            "sync_for" => $sync_for,
            "order" => [
                "id" => $summaryData['id'],
                "marketplace_primary_id" => $summaryData['marketplace_primary_id'],
                "marketplace_record_type" => $summaryData['marketplace_record_type'],
                "document_number" => $summaryData['document_number'],
                "invoice_date" => $summaryData['invoice_date'],
                "total_amount" => $summaryData['total_amount'],
                "json_amount" => $summaryData['json_amount'],
                "tax_type" => $summaryData['tax_type'],
                "tally_id" => $summaryData['tally_id'],
            ],
            "account_mapping_data" => $accountMapping,
            "journal_data" => $journalArray
        ];
        return $extraData;
    }

    public function createJournalSummaryXml($connectionDetails,$attributeArray,$setVoucherType,$journalArray,$journalItemArray,$summaryData,$sync_for)
    {
        $reqDataAry = [
            "HEADER" => [
                "VERSION" => "1",
                "TALLYREQUEST" => "IMPORT",
                "TYPE" => "DATA",
                "ID" => "Vouchers",
            ],
            "BODY" => [
                "DESC" => [
                    "STATICVARIABLES" => ["SVCURRENTCOMPANY" => $connectionDetails->company_name],
                ],
                "DATA" => [
                    "TALLYMESSAGE" => [
                        "VOUCHER" => [
                            "@attributes" => $attributeArray,
                            "VOUCHERTYPENAME" => $setVoucherType,//SET VOUCHER TYPE (SLAES, CREADIT NOTE, JOURNAL) GET BY FUNCTION
                            "DATE" => $journalArray['DATE'],
                            "REFERENCE" => $journalArray['REFERENCE'],
                            "NARRATION" => $journalArray['NARRATION'],
                            "EFFECTIVEDATE" => $journalArray['EFFECTIVEDATE'],
                            "ALLLEDGERENTRIES.LIST" => $journalItemArray,
                        ],
                    ],
                ],
            ],
        ];
        $extraData = $this->createJournalSummaryExtraData($connectionDetails,$summaryData,$journalArray,array(),$sync_for);
        return array($reqDataAry,$extraData);
    }


    public function createJournalIndividualExtraData($connectionDetails,$orderData,$journalArray,$accountMapping,$sync_for){
        
        $extraData = [
            "seller_id" => $connectionDetails['seller_id'],
            "module" => $connectionDetails['module'],
            "company" => $connectionDetails->company_name,
            "sync_for" => $sync_for,
            "name" => $connectionDetails->username,
            "order_id" => $orderData['id'],
            "order" => [
                "id" => $orderData['id'],
                "marketplace_primary_id" => $orderData['marketplace_order_id'],
                //"marketplace_record_type" => $orderData['marketplace_record_type'],
                "document_number" => $orderData['order_number'],
                "invoice_date" => $orderData['order_created_date'],
                "total_amount" => $orderData['total_amount'],
                //"json_amount" => $orderData['json_amount'],
                "tax_type" => $orderData['tax_type'],
                "tally_id" => $orderData['tally_id'],
            ],
            "account_mapping_data" => $accountMapping,
            "journal_data" => $journalArray
        ];
        return $extraData;
    }

    public function createJournalIndividualXml($connectionDetails,$attributeArray,$setVoucherType,$journalArray,$journalItemArray,$orderData,$accountMapping,$LEDGERENTRIES,$sync_for)
    {
        $reqDataAry = [
            "HEADER" => [
                "VERSION" => "1",
                "TALLYREQUEST" => "IMPORT",
                "TYPE" => "DATA",
                "ID" => "Vouchers",
            ],
            "BODY" => [
                "DESC" => [
                    "STATICVARIABLES" => ["SVCURRENTCOMPANY" => $connectionDetails->company_name],
                ],
                "DATA" => [
                    "TALLYMESSAGE" => [
                        "VOUCHER" => [
                            "@attributes" => $attributeArray,
                            "VOUCHERTYPENAME" => $setVoucherType,//SET VOUCHER TYPE (SLAES, CREADIT NOTE, JOURNAL) GET BY FUNCTION
                            "DATE" => $journalArray['DATE'],
                            "ISINVOICE" => 'Yes',
                            "REFERENCE" => $journalArray['REFERENCE'],
                            "NARRATION" => $journalArray['NARRATION'],
                            "EFFECTIVEDATE" => $journalArray['EFFECTIVEDATE'],
                            "LEDGERENTRIES.LIST" => $LEDGERENTRIES,
                            "ALLINVENTORYENTRIES.LIST" => $journalItemArray,
                        ],
                    ],
                ],
            ],
        ];
        $extraData = $this->createJournalIndividualExtraData($connectionDetails,$orderData,$journalArray,$accountMapping,$sync_for);
        return array($reqDataAry,$extraData);

    
    }

    public function formatLineItemData($lineItems){
        $lineItemsArray = [];
        $i = 0;

        //Handled input format to work rest code as it is.
        foreach ($lineItems as $parentKey => $parentValue) {
            if (!empty($parentValue['data'])) {
                foreach ($parentValue['data'] as $childKey => $childValue) {
                    if (!empty($childValue['data'])) {
                        foreach ($childValue['data'] as $taxKey => $taxValue) {
                            $amount = !empty($taxValue['amount']) ? $taxValue['amount'] : 0;
                            $tax_amount = !empty($taxValue['taxAmount']) ? $taxValue['taxAmount'] : 0;
                            $discount_amount = !empty($taxValue['discountAmount']) ? $taxValue['discountAmount'] : 0;
                            $lineItemsArray[$i] = array('key' => $taxKey, 'parent' => $childKey, 'superParent' => $parentKey, 'amount' => $amount, 'taxAmount' => $tax_amount, 'discountAmount' => $discount_amount);
                            if (!isset($taxValue['taxAmount'])) {
                                unset($lineItemsArray[$i]['taxAmount']);
                            }
                            $i++;
                        }
                    } else {
                        $amount = !empty($childValue['amount']) ? $childValue['amount'] : 0;
                        $tax_amount = !empty($childValue['taxAmount']) ? $childValue['taxAmount'] : 0;
                        $discount_amount = !empty($childValue['discountAmount']) ? $childValue['discountAmount'] : 0;
                        $lineItemsArray[$i] = array('key' => '', 'parent' => $childKey, 'superParent' => $parentKey, 'amount' => $amount, 'taxAmount' => $tax_amount, 'discountAmount' => $discount_amount);
                        if (!isset($childValue['taxAmount'])) {
                            unset($lineItemsArray[$i]['taxAmount']);
                        }
                        $i++;
                    }
                }
            } else {

                foreach ($parentValue as $childKey => $childValue) {

                    $amount = !empty($childValue['amount']) ? $childValue['amount'] : 0;
                    $tax_amount = !empty($childValue['taxAmount']) ? $childValue['taxAmount'] : 0;
                    $discount_amount = !empty($childValue['discountAmount']) ? $childValue['discountAmount'] : 0;
                    $lineItemsArray[$i] = array('key' => '', 'parent' => $childKey, 'superParent' => $parentKey, 'amount' => $amount, 'taxAmount' => $tax_amount, 'discountAmount' => $discount_amount);
                    if (!isset($childValue['taxAmount'])) {
                        unset($lineItemsArray[$i]['taxAmount']);
                    }
                    $i++;
                }
            }
        }

        return $lineItemsArray;
    }

    //JOURNAL CREATED OR UPDATE IN TALLY RESPONSE
    public static function JournalCreate($res)
    {   
        $response = json_decode($res['response_data']);
        $extraData = json_decode($res['extra_data']);

        if(isset($response->BODY->DATA->IMPORTRESULT) && ($response->BODY->DATA->IMPORTRESULT->CREATED == 1 || $response->BODY->DATA->IMPORTRESULT->ALTERED == 1)){
            $voucherId = $response->BODY->DATA->IMPORTRESULT->LASTVCHID;

            Tallysummary::updateSummaryData($voucherId, $extraData);
            list($tallyConnected, $tallySyncSetting) = Controller::getTallyConnection($extraData->seller_id, $extraData->module);
            Tallysummaryaccmappinglog::insertAccountMappingData($tallyConnected, $tallySyncSetting, $extraData->order->id, $extraData->account_mapping_data);
        } else if(isset($response->BODY->DATA->LINEERROR) && stripos($response->BODY->DATA->LINEERROR, 'does not exist!')){ //LEDGERNAME NOT EXITE IN TALLY
            $status = !empty($extraData->order->tally_id) ? "SyncPending": "Pending";
            Tallysummary::updateSummaryStatus((array)$extraData, $status);
        } else {
            $status = !empty($extraData->order->tally_id) ? "SyncPending": "Pending";
            Tallysummary::updateSummaryStatus((array)$extraData, $status);
            $actionlog = array(
                'error_code' => '400',
                'error_msg'  => "Journal voucher create error",
                'uniqueId'   => $extraData->summary_id,
                'data'       => array('action' => 'Error while create Journal voucher', 'order_data' => $extraData, 'tally_response' => $response),
                'seller_id'  => $extraData->seller_id,
                'module'     => $extraData->module
            );
            Apierrorlog::addErrorlog($actionlog);
        }
        return '{"success": 1, "message":"Success"}';

    }

    public static function invoiceCreate($res)
    {   
        $response = json_decode($res['response_data']);
        $extraData = json_decode($res['extra_data']);

        Log::info("Invoice DAta1".print_r( $response,1));
        Log::info("Invoice extraData".print_r( $extraData,1));
        if(isset($response->BODY->DATA->IMPORTRESULT) && ($response->BODY->DATA->IMPORTRESULT->CREATED == 1 || $response->BODY->DATA->IMPORTRESULT->ALTERED == 1)){
            $voucherId = $response->BODY->DATA->IMPORTRESULT->LASTVCHID;
            if($extraData->sync_for == 'refund')
            Tallyrefunds::updateRefundData($voucherId, $extraData);
            else
            TallyOrders::updateOrderData($voucherId, $extraData);
        } else if(isset($response->BODY->DATA->LINEERROR) && stripos($response->BODY->DATA->LINEERROR, 'does not exist!')){ //LEDGERNAME NOT EXITE IN TALLY
            $status = !empty($extraData->order->tally_id) ? "SyncPending": "Pending";
            if($extraData['sync_for'] == 'refund')
            Tallyrefunds::updateRefundStatus($voucherId, $extraData);
            else
            TallyOrders::updateOrderStatus((array)$extraData, $status);
        } else {
            $status = !empty($extraData->order->tally_id) ? "SyncPending": "Pending";
            if($extraData->sync_for == 'refund')
            Tallyrefunds::updateRefundStatus((array)$extraData, $status);
            else
            TallyOrders::updateOrderStatus((array)$extraData, $status);
            $actionlog = array(
                'error_code' => '400',
                'error_msg'  => "sales invoice create error",
                'uniqueId'   => $extraData->order_id,
                'data'       => array('action' => 'Error while create sales invoice', 'order_data' => $extraData, 'tally_response' => $response),
                'seller_id'  => $extraData->seller_id,
                'module'     => $extraData->module
            );
            Apierrorlog::addErrorlog($actionlog);
        }
        Log::info('VCFT flow');
        /*On Invoice sync fetch voucher details for more details*/
        TallyOrders::where('id',$extraData->order_id)->where('module', $extraData->module)->where('seller_id', $extraData->seller_id)->update(['sync_flag' => "VCFT"]);

        $orderObj = new InvoiceController();
        $orderObj->checkOrderExitsQueue($extraData->order_id);
    
        return '{"success": 1, "message":"Success"}';

    }

    //WHEN JOURNAL CREATED UPDATE SUMMARY TALLY ID IN SUMMARY TABLE
    public static function VoucherCollection($res)
    {   
        $response = json_decode($res['response_data']);
        $extraData = json_decode($res['extra_data']);
        Log::info("VoucherCollection Data".print_r( $response,1));
        Log::info("VoucherCollection extraData".print_r( $extraData,1));
        
        /*If voucher exist then sync and if not order create queue*/
        if(isset($response->BODY->DATA->COLLECTION->VOUCHER->MASTERID) && !is_null($response->BODY->DATA->COLLECTION->VOUCHER->MASTERID)){
            TallyOrders::where('id',$extraData->order_id)->where('seller_id',$extraData->seller_id)->where('module',$extraData->module)->update(['tally_id'=>$response->BODY->DATA->COLLECTION->VOUCHER->MASTERID,'sync_status'=>'Sync','tally_vchr_id'=>$response->BODY->DATA->COLLECTION->VOUCHER->VOUCHERNUMBER]);
            /*Check For Payment in db*/
            $tallyPayment = Tallypayments::where('seller_id',$extraData->seller_id)->where('module',$extraData->module)->where('tally_order_id',$extraData->order_id)->first();
            if(!empty($tallyPayment)){
                TallyOrders::where('id',$extraData->order_id)->where('module', $extraData->module)->where('seller_id', $extraData->seller_id)->update(['sync_flag' => "PS"]);
                $request = new \Illuminate\Http\Request();
                $request->replace(['seller_id' => $extraData->seller_id,'order_id'=>$extraData->order_id]);
                $orderObj = new ReceiptController();
                $orderObj->createReceipt($request);
            }
        }else{
            TallyOrders::where('id',$extraData->order_id)->where('module', $extraData->module)->where('seller_id', $extraData->seller_id)->update(['sync_flag' => "INV"]);
            /*Put order check queue*/
            $request = new \Illuminate\Http\Request();
            $request->replace(['sellerid' => $extraData->seller_id,'order_id'=>$extraData->order_id]);
            $orderObj = new InvoiceController();
            $orderObj->createOrderInvoice($request);
        }
        
        return '{"success": 1, "message":"Success"}';

    }

    public function getInvoiceXml($connectionDetails,$attributeArray,$setVoucherType,$orderData){
  
        $reqDataAry = [
            "HEADER" => [
                "VERSION" => "1",
                "TALLYREQUEST" => "Export",
                "TYPE" => "Collection",
                "ID" => "Voucher Collection",
            ],
            "BODY" => [
                "DESC" => [
                    "STATICVARIABLES" => [
                        "SVEXPORTFORMAT" => '$$SysName:XML',
                        "SVCURRENTCOMPANY" => $connectionDetails->company_name
                    ],
                    "TDL" => [
                        "TDLMESSAGE" => [
                            "COLLECTION" => [
                                "@attributes" => $attributeArray,
                                "TYPE" => 'Voucher',
                                "FETCH" =>"VOUCHERTYPENAME,VOUCHERNUMBER,MASTERID,REFERENCE,DATE,NARRATION,PERSISTEDVIEW,EFFECTIVEDATE,ALLINVENTORYENTRIES.LIST",
                                "FILTER"=>'TYPEOFFILTER'
                            ],
                            "SYSTEM" => [
                                "@attributes" => [
                                    "TYPE" => "Formulae",
                                    "NAME" => "TYPEOFFILTER",
                                ],
                                "@value" => '$VOUCHERTYPENAME Equal "'.$setVoucherType.'" AND $REFERENCE Equal "'.$orderData['document_number'].'"',
                            ],
                        ],
                    ],
                ],
            ],
        ];
        $extraData = $this->getInvoiceExtraData($connectionDetails,$attributeArray,$setVoucherType,$orderData);
        return array($reqDataAry,$extraData);
    }

    public function getInvoiceExtraData($connectionDetails,$attributeArray,$setVoucherType,$orderData){
        
        $extraData = [
            "seller_id" => $connectionDetails['seller_id'],
            "module" => $connectionDetails['module'],
            "company" => $connectionDetails->company_name,
            "name" => $connectionDetails->username,
            "order_id" => $orderData['id'],
            "order" => [
                "id" => $orderData['id'],
                "marketplace_primary_id" => $orderData['marketplace_order_id'],
                "document_number" => $orderData['order_number'],
                "invoice_date" => $orderData['order_created_date'],
                "total_amount" => $orderData['total_amount'],
                "tax_type" => $orderData['tax_type'],
                "tally_id" => $orderData['tally_id'],
            ]
        ];
        return $extraData;
    }
}
