<?php
namespace App\Http\Traits;
use App\Helpers\Helper;
use App\Models\Apierrorlog;
use App\Models\Tallypayments;
use App\Models\TallyOrders;
use Log;

/*
Manali Patel - 15-09-2023
This trait is used to form xml requests to create and fetch Receipts.
*/
trait Receipt
{
    public function createReceiptExtraData($connectionDetails,$receiptData,$receiptArray,$sync_for){
        
        $extraData = [
            "seller_id" => $connectionDetails['seller_id'],
            "module" => $connectionDetails['module'],
            "company" => $connectionDetails->company_name,
            "sync_for" => $sync_for,
            "name" => $connectionDetails->username,
            "receipt_id" => $receiptData['id'],
            "receipt" => [
                "id" => $receiptData['id'],
                "marketplace_primary_id" => $receiptData['tally_order_id'],
                "receipt_date" => $receiptData['payment_date'],
                "total_amount" => $receiptData['total_amount'],
                "tally_id" => $receiptData['tally_id'],
            ],
            "receipt_data" => $receiptArray
        ];
        return $extraData;
    }

    public function createReceiptXml($connectionDetails,$attributeArray,$receiptArray,$receiptData,$ALLLEDGERENTRIES,$sync_for = 'order')
    {
        $reqDataAry = [
            "HEADER" => [
                "VERSION" => "1",
                "TALLYREQUEST" => "IMPORT",
                "TYPE" => "DATA",
                "ID" => "Vouchers",
            ],
            "BODY" => [
                "DESC" => [
                    "STATICVARIABLES" => ["SVCURRENTCOMPANY" => $connectionDetails->company_name],
                ],
                "DATA" => [
                    "TALLYMESSAGE" => [
                        "VOUCHER" => [
                            "@attributes" => $attributeArray,
                            "VOUCHERTYPENAME" => 'Receipt',
                            "DATE" => $receiptArray['DATE'],
                            "PARTYLEDGERNAME" => $receiptArray['PARTYLEDGERNAME'],
                            "PERSISTEDVIEW" => 'Accounting Voucher View',
                            "REFERENCE" => $receiptArray['REFERENCE'],
                            "NARRATION" => $receiptArray['NARRATION'],
                            "EFFECTIVEDATE" => $receiptArray['EFFECTIVEDATE'],
                            "ALLLEDGERENTRIES.LIST" => $ALLLEDGERENTRIES,
                        ],
                    ],
                ],
            ],
        ];
        $extraData = $this->createReceiptExtraData($connectionDetails,$receiptData,$receiptArray,$sync_for);
        return array($reqDataAry,$extraData);
    }

    public static function receiptCreate($res)
    {   
        $response = json_decode($res['response_data']);
        $extraData = json_decode($res['extra_data']);

        Log::info("Receipt data".print_r( $response,1));
        Log::info("Receipt extraData".print_r( $extraData,1));
        if(isset($response->BODY->DATA->IMPORTRESULT) && ($response->BODY->DATA->IMPORTRESULT->CREATED == 1 || $response->BODY->DATA->IMPORTRESULT->ALTERED == 1)){
            $voucherId = $response->BODY->DATA->IMPORTRESULT->LASTVCHID;
            Tallypayments::updatePaymentData($voucherId, $extraData);
            if($extraData->sync_for == 'order'){
                TallyOrders::where('seller_id',$extraData->seller_id)->where('module',$extraData->module)->where('id',$extraData->receipt->marketplace_primary_id)->update(['sync_flag'=>'Y','sync_status'=>'Sync']);
            }
            elseif($extraData->sync_for == 'refund'){
                Tallyrefunds::where('seller_id',$extraData->seller_id)->where('module',$extraData->module)->where('id',$extraData->receipt->marketplace_primary_id)->update(['sync_flag'=>'Y','sync_status'=>'Sync']);
            }
        } else if(isset($response->BODY->DATA->LINEERROR) && stripos($response->BODY->DATA->LINEERROR, 'does not exist!')){ //LEDGERNAME NOT EXITE IN TALLY
            $status = !empty($extraData->order->tally_id) ? "SyncPending": "Pending";
            Tallypayments::updatePaymentStatus((array)$extraData, $status);
        } else {
            $status = !empty($extraData->order->tally_id) ? "SyncPending": "Pending";
            Tallypayments::updatePaymentStatus((array)$extraData, $status);
            $actionlog = array(
                'error_code' => '400',
                'error_msg'  => "sales invoice create error",
                'uniqueId'   => $extraData->order_id,
                'data'       => array('action' => 'Error while create sales invoice', 'order_data' => $extraData, 'tally_response' => $response),
                'seller_id'  => $extraData->seller_id,
                'module'     => $extraData->module
            );
            Apierrorlog::addErrorlog($actionlog);
        }
        return '{"success": 1, "message":"Success"}';

    }
}


