<?php

namespace App\Models;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\Eloquent\Model;

class TallyOrders extends Model
{
    protected $connection             = 'tally';
    protected $dates                  = [''];
    protected $table                  = 'tally_orders';
    protected $rules                  = array();
    protected $injectUniqueIdentifier = true;

    public static function fetchUnsyncOrders($tallyConnected, $request, $task = 'create', $ids = array())
    {
        $orderStatus = ($request->get('status') != '') ? (array) $request->get('status') : array('New','Sync');
        $orderLimit  = ($request->get('limit') != '') ? $request->get('limit') : config('mastertallyconstant.OrderSyncLimitFromDB');
        $orderNumber = ($request->get('order_number') != '') ? explode(",", $request->get('order_number')) : '';

        $orderObj = TallyOrders::where('seller_id', $tallyConnected['seller_id'])
            ->where('module', $tallyConnected['module'])
            ->orderBy('order_created_date', 'ASC');

        if (!empty($orderNumber)) {
            $orderObj = $orderObj->whereIn('order_number', $orderNumber);
        }else{
            if($task == 'create'){
                $orderLimit  = ($request->get('limit') != '') ? $request->get('limit') : config('mastertallyconstant.OrderSyncLimitFromDB');
                $orderObj = $orderObj
                ->where('sync_flag','!=','Y')->whereIn('sync_status', $orderStatus)->limit($orderLimit);
            }
        }

        if(!empty($ids)){
            $orderObj = $orderObj->whereNotIn('id', $ids);
        }
        //using pluck get all order_id in array
        $orderIdsObj = $orderObj->pluck('id')->toArray();
        $orderObj    = $orderObj->get()->toArray();

        if (!empty($orderObj)) {
            $orderLineItems = TallyOrderLineItems::whereIn('tally_order_id', $orderIdsObj)
                ->where('seller_id', $tallyConnected['seller_id'])
                ->where('module', $tallyConnected['module'])
                ->orderByRaw("FIELD(line_item_type, 'Product', 'Description', 'Giftcard', 'Shipping')")
                ->whereNull('deleted_at')->get()->toArray();

            //Assign line items to order
            foreach ($orderObj as $key => $order) {
                $lineItemKeys = array_keys(array_column($orderLineItems, 'tally_order_id'), $order['id']);
                if (!empty($lineItemKeys)) {
                    foreach ($lineItemKeys as $lineItemIndex) {
                        $orderObj[$key]['line_items'][] = $orderLineItems[$lineItemIndex];
                    }
                } else {
                    //If line item is not found then mark that order as SNR. Front end if line item total amount is mismatch then it will do SNR to New. NS:23-11-2022
                    TallyOrders::where('seller_id', $tallyConnected['seller_id'])->where('module', $tallyConnected['module'])->where('id', $orderObj[$key])->update(['sync_status' => 'SNR']);
                    unset($orderObj[$key]); //Do not process that order if no line item
                }
            }
        }

        return $orderObj;
    }

    public static function getIndividualOrder($tallyConnected, $order_id){
        $orderDetailsObj = TallyOrders::where('seller_id', $tallyConnected['seller_id'])
            ->where('module', $tallyConnected['module'])->where('id',$order_id)->get()->toArray();

        if (!empty($orderDetailsObj)) {
            $orderLineItems = TallyOrderLineItems::where('tally_order_id', $order_id)
                ->where('seller_id', $tallyConnected['seller_id'])
                ->where('module', $tallyConnected['module'])
                ->orderByRaw("FIELD(line_item_type, 'Product', 'Description', 'Giftcard', 'Shipping')")
                ->whereNull('deleted_at')->get()->toArray();

            //Assign line items to order
            foreach ($orderDetailsObj as $key => $order) {
                $lineItemKeys = array_keys(array_column($orderLineItems, 'tally_order_id'), $order['id']);
                if (!empty($lineItemKeys)) {
                    foreach ($lineItemKeys as $lineItemIndex) {
                        $orderDetailsObj[$key]['line_items'][] = $orderLineItems[$lineItemIndex];
                    }
                } else {
                    unset($orderDetailsObj[$key]); //Do not process that order if no line item
                }
            }
        }

        return $orderDetailsObj;            
    }

    public static function resetTallyDetails($tallyConnected, $orderData){
        TallyOrders::where('id', $orderData['id'])->where('seller_id', $tallyConnected['seller_id'])->where('module', $tallyConnected['module'])->update([
            'tally_id'             => null,
            'tally_amount'         => null,
            'tally_invoice_number' => null,
            'tally_currency_rate'  => null,
            'sync_status'       => 'New',
            'sync_flag'         => 'N'
        ]);
    }

    public static function markOrderPending($tallyConnected, $orderData)
    {
        $status = !empty($orderData['tally_id']) ? 'SyncPending' : 'Pending';
        TallyOrders::where('id', $orderData['id'])->where('seller_id', $tallyConnected['seller_id'])->where('module', $tallyConnected['module'])->update([
            'sync_status'       => $status
        ]);
    }

    public static function updateOrderData($voucherId, $extraData)
    {
        TallyOrders::where('id',$extraData->order_id)->where('module', $extraData->module)->where('seller_id', $extraData->seller_id)->update(['tally_id' => $voucherId, 'sync_flag' => 'Y', 'sync_status' => 'Sync']);
    }

    public static function checkOrderExists($tallyData, $extraData)
    {
        $summaryData =  Tallysummary::where('id', $extraData->summary_id)->where('module', $extraData->module)->where('seller_id', $extraData->seller_id)->first();
        if(!empty($tallyData) && (Controller::DateWithFormat($tallyData->BODY->DATA->COLLECTION->VOUCHER->DATE,'Y-m-d') == Controller::DateWithFormat($summaryData['invoice_date'],'Y-m-d')) && $tallyData->BODY->DATA->COLLECTION->VOUCHER->REFERENCE == $summaryData['document_number']){
            Tallysummary::updateSummaryData($tallyData->BODY->DATA->COLLECTION->VOUCHER->MASTERID, $extraData);
        }else{
            //status pending by summary
            $status = !empty($extraData->order->tally_id) ? "SyncPending": "Pending";
            Tallysummary::updateSummaryStatus((array)$extraData, $status);
            $actionlog = array(
                'error_code' => '400',
                'error_msg'  => 'Journal with reference already exists',
                'uniqueId'   => $summaryData['id'],
                'data'       => array('action' => 'Journal Already Exist', 'data' => $tallyData),
                'seller_id'  => $extraData->seller_id,
                'module'  => $extraData->module
            );
            Apierrorlog::addErrorlog($actionlog);
        }
    }

    public static function updateOrderStatus($data, $status)
    {
        TallyOrders::where('id',$data['order_id'])->where('module', $data['module'])->where('seller_id', $data['seller_id'])->update(['sync_status' => $status]);
    }
}
