<?php

namespace App\Models;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\Eloquent\Model;

class Tallypayouts extends Model
{
    protected $connection             = 'tally';
    protected $dates                  = [''];
    protected $table                  = 'tally_payout';
    protected $rules                  = array();
    protected $injectUniqueIdentifier = true;

    public static function fetchUnsyncPayouts($tallyConnected, $request, $task = 'create', $ids = array())
    {
        $orderStatus = ($request->get('status') != '') ? (array) $request->get('status') : array('New','Sync');
        $orderLimit  = ($request->get('limit') != '') ? $request->get('limit') : config('mastertallyconstant.OrderSyncLimitFromDB');
        $payout_id = ($request->get('payout_id') != '') ? explode(",", $request->get('payout_id')) : '';

        $payoutObj = Tallypayouts::where('seller_id', $tallyConnected['seller_id'])
            ->where('module', $tallyConnected['module'])
            ->orderBy('payout_date', 'ASC');

        if (!empty($payout_id)) {
            $payoutObj = $payoutObj->whereIn('payout_id', $payout_id);
        }else{
            if($task == 'create'){
                $orderLimit  = ($request->get('limit') != '') ? $request->get('limit') : config('mastertallyconstant.OrderSyncLimitFromDB');
                $payoutObj = $payoutObj
                ->where(function($query){
                            $query->where('sync_flag','N');
                            $query->orWhere('sync_flag', 'P');
                        })->whereIn('sync_status', $orderStatus)->limit($orderLimit);
            } else {
                $orderLimit  = ($request->get('limit') != '') ? $request->get('limit') : config('mastertallyconstant.OrderUpdateSyncLimitFromDB');
                $payoutObj = $payoutObj->where(function($query){
                            $query->where('sync_flag','N');
                            $query->orWhere('sync_flag', 'P');
                        })->where('sync_status', 'Sync')->whereNotNull('tally_id')->limit($orderLimit);
            }
        }

        if(!empty($ids)){
            $payoutObj = $payoutObj->whereNotIn('id', $ids);
        }
        //using pluck get all order_id in array
        $orderIdsObj = $payoutObj->pluck('id')->toArray();
        $payoutObj    = $payoutObj->get()->toArray();

        if (!empty($payoutObj)) {
            $orderLineItems = Tallypayoutlineitems::whereIn('tally_payout_id', $orderIdsObj)
                ->where('seller_id', $tallyConnected['seller_id'])
                ->where('module', $tallyConnected['module'])
                ->whereNull('deleted_at')->get()->toArray();

            //Assign line items to order
            foreach ($payoutObj as $key => $order) {
                $lineItemKeys = array_keys(array_column($orderLineItems, 'tally_payout_id'), $order['id']);
                if (!empty($lineItemKeys)) {
                    foreach ($lineItemKeys as $lineItemIndex) {
                        $payoutObj[$key]['line_items'][] = $orderLineItems[$lineItemIndex];
                    }
                } else {
                    Tallypayouts::where('seller_id', $tallyConnected['seller_id'])->where('module', $tallyConnected['module'])->where('id', $payoutObj[$key])->update(['sync_status' => 'SNR']);
                    unset($payoutObj[$key]); //Do not process that order if no line item
                }
            }
        }

        return $payoutObj;
    }

    public static function updatePayoutData($voucherId, $extraData)
    {
        Tallypayouts::where('id',$extraData->order_id)->where('module', $extraData->module)->where('seller_id', $extraData->seller_id)->update(['tally_id' => $voucherId, 'sync_flag' => 'Y', 'sync_status' => 'Sync']);
    }

    public static function updatePayoutStatus($data, $status)
    {
        Tallypayouts::where('id',$data['order_id'])->where('module', $data['module'])->where('seller_id', $data['seller_id'])->update(['sync_status' => $status]);
    }
}
