<?php

namespace App\Models;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\Eloquent\Model;

class Tallyrefunds extends Model
{
    protected $connection             = 'tally';
    protected $dates                  = [''];
    protected $table                  = 'tally_order_refund';
    protected $rules                  = array();
    protected $injectUniqueIdentifier = true;

    public static function fetchUnsyncRefunds($tallyConnected, $request, $task = 'create', $ids = array())
    {
        $RefundStatus = ($request->get('status') != '') ? (array) $request->get('status') : array('New','Sync');
        $refundLimit  = ($request->get('limit') != '') ? $request->get('limit') : config('mastertallyconstant.OrderSyncLimitFromDB');
        $refundNumber = ($request->get('order_number') != '') ? explode(",", $request->get('order_number')) : '';

        $refundObj = Tallyrefunds::select('tally_order_refund.amount as total_amount','tally_order_refund.*')->where('seller_id', $tallyConnected['seller_id'])
            ->where('module', $tallyConnected['module'])
            ->orderBy('refund_date', 'ASC');

        if (!empty($refundNumber)) {
            $refundObj = $refundObj->whereIn('order_number', $refundNumber);
        }else{
            if($task == 'create'){
                $refundLimit  = ($request->get('limit') != '') ? $request->get('limit') : config('mastertallyconstant.OrderSyncLimitFromDB');
                $refundObj = $refundObj
                ->where(function($query){
                            $query->where('sync_flag','N');
                            $query->orWhere('sync_flag', 'P');
                        })->whereIn('sync_status', $RefundStatus)->limit($refundLimit);
            } else {
                $refundLimit  = ($request->get('limit') != '') ? $request->get('limit') : config('mastertallyconstant.OrderUpdateSyncLimitFromDB');
                $refundObj = $refundObj->where(function($query){
                            $query->where('sync_flag','N');
                            $query->orWhere('sync_flag', 'P');
                        })->where('sync_status', 'Sync')->whereNotNull('tally_id')->limit($refundLimit);
            }
        }

        if(!empty($ids)){
            $refundObj = $refundObj->whereNotIn('id', $ids);
        }
        //using pluck get all order_id in array
        $refundIdsObj = $refundObj->pluck('id')->toArray();
        $refundObj    = $refundObj->get()->toArray();

        if (!empty($refundObj)) {
            $refundLineItems = Tallyrefundlineitems::whereIn('tally_refund_id', $refundIdsObj)
                ->where('seller_id', $tallyConnected['seller_id'])
                ->where('module', $tallyConnected['module'])
                ->orderByRaw("FIELD(line_item_type, 'Product', 'Description', 'Giftcard', 'Shipping')")
                ->whereNull('deleted_at')->get()->toArray();

            //Assign line items to order
            foreach ($refundObj as $key => $refund) {
                $lineItemKeys = array_keys(array_column($refundLineItems, 'tally_refund_id'), $refund['id']);
                if (!empty($lineItemKeys)) {
                    foreach ($lineItemKeys as $lineItemIndex) {
                        $refundObj[$key]['line_items'][] = $refundLineItems[$lineItemIndex];
                    }
                } else {
                    //If line item is not found then mark that order as SNR. Front end if line item total amount is mismatch then it will do SNR to New. NS:23-11-2022
                    Tallyrefunds::where('seller_id', $tallyConnected['seller_id'])->where('module', $tallyConnected['module'])->where('id', $refundObj[$key])->update(['sync_status' => 'SNR']);
                    unset($refundObj[$key]); //Do not process that order if no line item
                }
            }
        }
        
        return $refundObj;
    }

    public static function getIndividualOrder($tallyConnected, $refund_id){
        $refundDetailsObj = Tallyrefunds::where('seller_id', $tallyConnected['seller_id'])
            ->where('module', $tallyConnected['module'])->where('id',$refund_id)->get()->toArray();

        if (!empty($refundDetailsObj)) {
            $refundLineItems = Tallyrefundlineitems::where('tally_order_id', $refund_id)
                ->where('seller_id', $tallyConnected['seller_id'])
                ->where('module', $tallyConnected['module'])
                ->orderByRaw("FIELD(line_item_type, 'Product', 'Description', 'Giftcard', 'Shipping')")
                ->whereNull('deleted_at')->get()->toArray();

            //Assign line items to order
            foreach ($refundDetailsObj as $key => $refund) {
                $lineItemKeys = array_keys(array_column($refundLineItems, 'tally_order_id'), $refund['id']);
                if (!empty($lineItemKeys)) {
                    foreach ($lineItemKeys as $lineItemIndex) {
                        $refundDetailsObj[$key]['line_items'][] = $refundLineItems[$lineItemIndex];
                    }
                } else {
                    unset($refundDetailsObj[$key]); //Do not process that order if no line item
                }
            }
        }

        return $refundDetailsObj;            
    }

    public static function resetTallyDetails($tallyConnected, $refundData){
        Tallyrefunds::where('id', $refundData['id'])->where('seller_id', $tallyConnected['seller_id'])->where('module', $tallyConnected['module'])->update([
            'tally_id'             => null,
            'tally_amount'         => null,
            'tally_invoice_number' => null,
            'tally_currency_rate'  => null,
            'sync_status'       => 'New',
            'sync_flag'         => 'N'
        ]);
    }

    public static function markRefundPending($tallyConnected, $refundData)
    {
        $status = !empty($refundData['tally_id']) ? 'SyncPending' : 'Pending';
        Tallyrefunds::where('id', $refundData['id'])->where('seller_id', $tallyConnected['seller_id'])->where('module', $tallyConnected['module'])->update([
            'sync_status'       => $status
        ]);
    }

    public static function updateRefundData($voucherId, $extraData)
    {
        Tallyrefunds::where('id',$extraData->order_id)->where('module', $extraData->module)->where('seller_id', $extraData->seller_id)->update(['tally_id' => $voucherId, 'sync_flag' => 'Y', 'sync_status' => 'Sync']);
    }

    public static function updateRefundStatus($data, $status)
    {
        Tallyrefunds::where('id',$data['order_id'])->where('module', $data['module'])->where('seller_id', $data['seller_id'])->update(['sync_status' => $status]);
    }
    
}
