<?php

namespace App\Models;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\Eloquent\Model;

class Tallysummary extends Model
{
    protected $connection             = 'tally';
    protected $dates                  = [''];
    protected $table                  = 'tally_summary';
    protected $rules                  = array();
    protected $injectUniqueIdentifier = true;

    public static function fetchUnsyncSummary($tallyConnected, $request)
    {
        $summaryStatus = ($request->get('status') != '') ? (array) $request->get('status') : array('New', 'Sync');
        $summaryLimit  = ($request->get('limit') != '') ? $request->get('limit') : 2;
        $summaryNumber = ($request->get('summary_id') != '') ? explode(",", $request->get('summary_id')) : '';
        
        $summaryObj = Tallysummary::where('seller_id', $tallyConnected['seller_id'])
            ->where('module', $tallyConnected['module'])
            ->orderBy('id', 'ASC')
            ->limit($summaryLimit);
        
        if (!empty($summaryNumber)) {
            $summaryObj = $summaryObj->whereIn('id', $summaryNumber);
        } else {
            //To avoid create batch of create and update invoice together from cron
            if($request->get('task') == 'update'){
                $summaryObj = $summaryObj->where('sync_flag', 'U');
            } else {
                $summaryObj = $summaryObj->where('sync_flag', 'N');
            }
            $summaryObj = $summaryObj->whereIn('sync_status', $summaryStatus);

        }

        $summaryObj = $summaryObj->get();
        return $summaryObj;
    }

    public static function getTotalJsonDiffRows($tallyConnected){
        return Tallysummary::select('*')
                    ->where('seller_id', $tallyConnected['seller_id'])
                    ->where('module', $tallyConnected['module'])
                    ->whereNull('tally_id')
                    ->where(DB::raw('(total_amount - json_amount)'), '>' , '1')
                    ->get();
    }

    public static function markOrderPending($tallyConnected, $ids) {
        $dbObj = Tallysummary::select('*')
            ->where('seller_id', $tallyConnected['seller_id'])
            ->where('module', $tallyConnected['module'])
            ->where('sync_status', '!=','Pending');
        
        if(!empty($ids))
            $dbObj = $dbObj->whereIn('id', explode(",",$ids));

        $dbObj = $dbObj->whereNull('tally_id')
            ->update(['sync_status'=>'Pending']);
    }

    public static function updateSummaryData($voucherId, $extraData)
    {
        Tallysummary::where('id',$extraData->summary_id)->where('module', $extraData->module)->where('seller_id', $extraData->seller_id)->update(['tally_id' => $voucherId, 'sync_flag' => 'Y', 'sync_status' => 'Sync']);
    }

    public static function checkSummaryExists($tallyData, $extraData)
    {
        $summaryData =  Tallysummary::where('id', $extraData->summary_id)->where('module', $extraData->module)->where('seller_id', $extraData->seller_id)->first();
        if(!empty($tallyData) && (Controller::DateWithFormat($tallyData->BODY->DATA->COLLECTION->VOUCHER->DATE,'Y-m-d') == Controller::DateWithFormat($summaryData['invoice_date'],'Y-m-d')) && $tallyData->BODY->DATA->COLLECTION->VOUCHER->REFERENCE == $summaryData['document_number']){
            Tallysummary::updateSummaryData($tallyData->BODY->DATA->COLLECTION->VOUCHER->MASTERID, $extraData);
        }else{
            //status pending by summary
            $status = !empty($extraData->order->tally_id) ? "SyncPending": "Pending";
            Tallysummary::updateSummaryStatus((array)$extraData, $status);
            $actionlog = array(
                'error_code' => '400',
                'error_msg'  => 'Journal with reference already exists',
                'uniqueId'   => $summaryData['id'],
                'data'       => array('action' => 'Journal Already Exist', 'data' => $tallyData),
                'seller_id'  => $extraData->seller_id,
                'module'  => $extraData->module
            );
            Apierrorlog::addErrorlog($actionlog);
        }
    }

    public static function updateSummaryStatus($data, $status)
    {
        Tallysummary::where('id',$data['summary_id'])->where('module', $data['module'])->where('seller_id', $data['seller_id'])->update(['sync_status' => $status]);
    }
}
